"""Provides Basic class and classes object.
"""

__docformat__ = 'restructuredtext'
__all__ = ['classes', 'Expr', 'defined_functions', 'DefinedFunction',
           'objects', 'Pair', 'IntegerList']

import sys
import types
import inspect
import textwrap

using_C_Expr = False
try:
    from . import expr_ext as expr_module
    using_C_Expr = True
except ImportError, msg:
    msg = str(msg)
    if msg!='No module named expr_ext' and msg!='cannot import name expr_ext':
        print msg
    from . import expr as expr_module

Expr = expr_module.Expr
Pair = expr_module.Pair

# Pickling support:
def _reconstruct(version, state):
    # Returned by <Expr instance>.__reduce__
    if version==1:
        cls, (head, data), hashvalue = state
        head = get_head(head)
        obj = cls(head, data)
        obj._sethash(hashvalue)
        return obj
    elif version==2:
        from .utils import get_head
        cls, (head, data), hashvalue = state
        if type(cls) is tuple:
            cls = cls[0](*cls[1])
        head = get_head(head)
        obj = cls(head, data)
        obj._sethash(hashvalue)
        return obj
    elif version==3:
        cls, (head, data), hashvalue = state
        if type(cls) is tuple:
            cls = cls[0](*cls[1])
        head = getattr(head, 'as_unique_head', lambda head=head: head)()
        obj = cls(head, data)
        obj._sethash(hashvalue)
        return obj
    raise NotImplementedError('pickle _reconstruct version=%r' % (version))

if using_C_Expr:
    # To add support pickling pure Expr instances, uncomment the
    # following line (but it is hackish):
    #
    #__builtins__['Expr'] = Expr
    #
    # Pickling Python classes derived from Expr should work fine
    # without this hack.
    pass

def get_base_attribute(cls, name):
    """
    Get attribute from cls base classes.
    """
    if name in cls.__dict__:
        return cls, cls.__dict__[name]
    for base in cls.__bases__:
        r = get_base_attribute(base, name)
        if r is not None:
            return r
    return

def inheritdoc(obj, cls):
    """
    If obj is a function intance then update its documentation string
    from the corresponding function documentation string defined in
    cls or one of its base classes.

    Returns obj.
    """
    if isinstance(obj, types.FunctionType):
        base_cls, base_func, base_doc = None, None, None
        for base in cls.__bases__:
            r = get_base_attribute(base, obj.__name__)
            if r is not None:
                base_cls, base_func = r
                base_doc = base_func.__doc__
                break
        if base_doc is not None:
            doc = obj.__doc__ or ''
            title = '### doc-string inherited from %s.%s ###'\
                    % (base_cls.__name__, obj.__name__)
            if title not in doc:
                obj.__doc__ = '%s\n%s\n%s' % (textwrap.dedent(doc),
                                              title,
                                              textwrap.dedent(base_doc).strip()) 
    return obj

def fcopy(func, name):
    """
    Return a copy of a function func with a new name.
    """
    return types.FunctionType(func.func_code, func.func_globals, name, func.func_closure)

class MetaCopyMethods(type):
    """
    Metaclass that makes copies of class methods that are defined
    by equality. For example,

    class A:
        __metaclass__ = MetaCopyMethods
        def foo(self):
            pass

        bar = foo

    is equivalent to

    class A:
        def foo(self):
            pass

        bar = fcopy(foo, 'bar')
    """
    def __init__(cls, name, bases, dict):
        cls.copy_methods(dict)
        type.__init__(cls, name, bases, dict)

    def copy_methods(cls, dict):
        for name in dict.keys():
            attr = dict[name]
            if isinstance(attr, types.FunctionType):
                if name != attr.__name__:
                    dict[name] = fcopy(attr, name)

class MetaInheritDocs(type):
    """
    Metaclass that sets undefined documentation strings of methods to
    the documentation strings of base class methods. For example,

    class A:
        def foo(self):
            '''foo doc'''

    class B(A):
        __metaclass__ = MetaCopyMethods
        def foo(self):
            pass

    is equivalent to

    class A:
        def foo(self):
            '''foo doc'''

    class B(A):
        def foo(self):
            '''### doc-string inherited from A.foo ###
            foo doc
            '''
    """

    def __init__(cls, name, bases, dict):
        type.__init__(cls, name, bases, dict)
        cls.inherit_docs(dict)

    def inherit_docs(cls, dict):
        for name in dict.keys():
            attr = dict[name]
            if hasattr(attr, '__doc__'):
                inheritdoc(attr, cls)

class MetaCopyMethodsInheritDocs(MetaCopyMethods, MetaInheritDocs):
    """
    A composite of MetaCopyMethods and MetaInheritDocs meta classes.
    """
    def __init__(cls, name, bases, dict):
        cls.copy_methods(dict)
        type.__init__(cls, name, bases, dict)
        cls.inherit_docs(dict)

class Holder:
    """ Holds pairs ``(name, value)`` as instance attributes.
    
    The set of Holder pairs is extendable by

    ::
    
      <Holder instance>.<name> = <value>

    and the values are accessible as

    ::

      value = <Holder instance>.<name>
    """
    def __init__(self, descr):
        self._descr = descr
        self._counter = 0

    def __str__(self):
        return self._descr % (self.__dict__)

    def __repr__(self):
        return '%s(%r)' % (self.__class__.__name__, str(self))
    
    def __setattr__(self, name, obj):
        if not self.__dict__.has_key(name) and self.__dict__.has_key('_counter'):
            self._counter += 1
        self.__dict__[name] = obj

    def iterNameValue(self):
        for k,v in self.__dict__.iteritems():
            if k.startswith('_'):
                continue
            yield k,v

classes = Holder('SympyCore classes holder (%(_counter)s classes)')
objects = Holder('SympyCore objects holder (%(_counter)s classes)')
heads = Holder('SympyCore expression heads holder (%(_counter)s heads)')
heads_precedence = Holder('SympyCore heads precedence holder (%(_counter)s items).')
defined_functions = Holder('SympyCore defined functions holder (%(_counter)s classes)')

class FunctionType(type):
    """ Metaclass to Function class.

    FunctionType implements the following features:

    1. If a class derived from ``Function`` has a name containing
       substring ``Function`` then the class will be saved as an
       attribute to ``classes`` holder. Such classes are assumed to
       be base classes to defined functions.

    2. Otherwise, ``Function`` subclasses are saved as attributes to
       ``defined_functions`` holder.

    """

    def __new__(typ, name, bases, attrdict):
        cls = type.__new__(typ, name, bases, attrdict)
        if 'Function' in name:
            setattr(classes, name, cls)
        else:
            setattr(defined_functions, name, cls)
        return cls

class DefinedFunction(object):
    """ Base class to symbolic functions.
    """
    __metaclass__ = FunctionType

    @classmethod
    def derivative(cls, arg):
        """ Return derivative function of cls at arg.
        """
        raise NotImplementedError(`cls, arg`)

    @classmethod
    def get_argument_algebras(cls):
        raise NotImplementedError(`cls`)

def get_nargs(obj):
    assert callable(obj),`obj`
    if isinstance(obj, classes.FunctionRing):
        return obj.nargs
    if inspect.isclass(obj):
        return len(inspect.getargspec(obj.__new__)[0])-1
    if inspect.isfunction(obj):
        return len(inspect.getargspec(obj)[0])
    if inspect.ismethod(obj):
        return len(inspect.getargspec(obj)[0]) - 1
    raise NotImplementedError(`obj, type(obj)`)        

classes.Expr = Expr
classes.Pair = Pair

class SymbolicEquality:
    """ Contex for logical operations.

    In the ``SymbolicEquality(<Algebra class>)`` context relational
    operations return ``Logic`` instances instead of computing
    lexicographic value of relational operations.

    For example,

    >>> x = Calculus('x')
    >>> with SymbolicEquality(Calculus):
    >>>     print x==x
    ...     
    ...     
    x==x
    >>> print x==x
    True

    Add ``from __future__ import with_statement`` to the header of
    python file when using Python version 2.5.
    
    """

    def __init__(self, *classes):
        self.classes = classes

    def __enter__(self):
        for cls in self.classes:
            cls.enable_symbolic_comparison()

    def __exit__(self, type, value, tb):
        for cls in self.classes:
            cls.disable_symbolic_comparison()
        return tb is None

class UnevaluatedAddition:
    """ Contex for not evaluating additive group operations.

    For example,

    >>> x = Calculus('x')
    >>> with UnevaluatedAddition(AdditiveGroup):
    >>>     print x - x
    ...     
    ...     
    x - x
    >>> print x - x
    0

    Add ``from __future__ import with_statement`` to the header of
    python file when using Python version 2.5.
    
    """

    def __init__(self, *classes):
        self.classes = classes

    def __enter__(self):
        for cls in self.classes:
            cls.algebra_options['evaluate_addition'] = False
            cls.algebra_options['evaluate_multiplication'] = False

    def __exit__(self, type, value, tb):
        for cls in self.classes:
            cls.algebra_options['evaluate_addition'] = True
            cls.algebra_options['evaluate_multiplication'] = True
        return tb is None

class InitModule:
    
    """ Holds a list of functions (or callable objects), composed by
    that are called exactly once by the execute methods. Functions are
    assumed to take one argument holding a module object. InitModule
    instance can be used as a decorator or functions can be added to
    the list using the register method.
    """

    def __init__(self):
        self.func_list = []
        
    def register(self, init_module_func):
        """ Register a function for execution.
        """
        if callable(init_module_func):
            self.func_list.append(init_module_func)
        else:
            raise NotImplementedError(`init_module_func`)

    def execute(self):
        """ Execute registered functions and discard them.
        """
        while self.func_list:
            func = self.func_list.pop(0)
            module_name = func.__module__
            module = sys.modules[module_name]
            #print 'Executing %s(%s):' % (func.__name__, module.__name__)
            func(module)

    def __call__(self, func):
        """ Register function via decorating it.
        """
        self.register(func)

    def import_heads(self):
        """Registers a function that adds heads symbols to a calling
        module.
        """
        frame = sys._getframe(1)
        module_name = frame.f_locals['__name__']

        def _import_heads(module):
            from sympycore.core import heads
            for n,h in heads.iterNameValue():
                setattr(module, n, h)

        _import_heads.__module__ = module_name

        self.register(_import_heads)

    def import_lowlevel_operations(self):
        """Registers a function that adds lowlevel operation functions
        like dict_get_item, dict_add_item, etc to a calling module.
        """
        frame = sys._getframe(1)
        module_name = frame.f_locals['__name__']
        
        def _import_lowlevel_operations(module):
            for n in dir(expr_module):
                if (('dict' in n or 'new' in n) and not n.startswith('_')) \
                       or n in ['add', 'mul', 'term_coeff']:
                    setattr(module, n, getattr(expr_module, n))
            module.IntegerList = IntegerList
            
        _import_lowlevel_operations.__module__ = module_name

        self.register(_import_lowlevel_operations)

    def import_numbers(self):
        """Registers a function that adds numbers and number types
        collections to a calling module.
        """
        frame = sys._getframe(1)
        module_name = frame.f_locals['__name__']

        def _import_numbers(module):
            from sympycore.arithmetic import numbers
            from sympycore.arithmetic import number_theory
            module.numbertypes = numbers.numbertypes
            module.numbertypes_set = numbers.numbertypes_set
            module.inttypes = numbers.inttypes
            module.inttypes_set = numbers.inttypes_set
            module.rationaltypes = numbers.rationaltypes
            module.realtypes = numbers.realtypes
            module.complextypes = numbers.complextypes
            module.Infinity = numbers.Infinity
            module.number_div = numbers.number_div
            module.mpq = numbers.mpq
            module.try_power = numbers.try_power
            module.gcd = number_theory.gcd
            
        _import_numbers.__module__ = module_name

        self.register(_import_numbers)

init_module = InitModule()
init_module.register(expr_module.init_module)

init_module.import_heads()

class IntegerList(Expr):
    """ IntegerList holds a list of integers and supports element-wise
    arithmetic operations.
    """

    @classmethod
    def convert(cls, arg, typeerror=True):
        t = type(arg)
        if t is int or t is long:
            return cls(INTEGER_LIST, [arg])
        if t is list:
            return cls(INTEGER_LIST, arg)
        if t is tuple:
            return cls(INTEGER_LIST, list(arg))
        if t is IntegerList:
            return arg
        if typeerror:
            raise TypeError('failed to convert %r to IntegerList' % (arg))
        return NotImplemented

    def __repr__(self):
        return '%s(%s)' % (type(self).__name__, self.data)

    def __len__(self): return len(self.data)

    def copy(self):
        return type(self)(self.data[:])

    def __getitem__(self, index):
        return self.data[index]

    def __setitem__(self, index, value):
        if self.is_writable:
            self.data[index] = value
        else:
            raise TypeError('this IntegerList instance is not writable')
        
    def __pos__(self):
        return self

    def __neg__(self):
        return IntegerList(INTEGER_LIST, [-i for i in self.data])

    def __add__(self, other):
        lhead, ldata = self.pair
        t = type(other)
        if t is int or t is long:
            if other==0:
                return self
            return IntegerList(INTEGER_LIST, [i + other for i in ldata])
        if t is IntegerList:
            rdata = other.data
        elif t is list or t is tuple:
            rdata = other
        else:
            return NotImplemented
        if len(ldata)!=len(rdata):
            raise TypeError('IntegerList.__add__ operands must have the same size')
        return IntegerList(INTEGER_LIST, [i + j for i,j in zip(ldata, rdata)])

    __radd__ = __add__

    def __sub__(self, other):
        t = type(other)
        if t is int or t is long:
            if other==0:
                return self
            return IntegerList(INTEGER_LIST, [i - other for i in self.data])
        ldata = self.data
        if t is IntegerList:
            rdata = other.data
        elif t is list or t is tuple:
            rdata = other
        else:
            return NotImplemented
        if len(ldata)!=len(rdata):
            raise TypeError('IntegerList.__sub__ operands must have the same size')
        return IntegerList(INTEGER_LIST, [i - j for i,j in zip(ldata, rdata)])

    def __rsub__(self, other):
        return (-self) + other

    def __mul__(self, other):
        t = type(other)
        if t is int or t is long:
            if other==1:
                return self
            return IntegerList(INTEGER_LIST, [i * other for i in self.data])
        ldata = self.data
        if t is IntegerList:
            rdata = other.data
        elif t is list or t is tuple:
            rdata = other
        else:
            return NotImplemented
        if len(ldata)!=len(rdata):
            raise TypeError('IntegerList.__mul__ operands must have the same size')
        return IntegerList(INTEGER_LIST, [i * j for i,j in zip(ldata, rdata)])

    __rmul__ = __mul__






from sympycore.core import Expr, Pair, heads, IntegerList
from sympycore.heads import *

class MyExpr(Expr):

    @classmethod
    def convert(cls, obj, typeerror=True):
        if isinstance(obj, cls):
            return obj
        if isinstance(obj, (int, long, float, complex)):
            return MyExpr(NUMBER, obj)
        if isinstance(obj, str):
            return MyExpr(SYMBOL, obj)
        if typeerror:
            raise TypeError('Cannot convert type %r to %r' % (type(obj), cls.__name__))
        return NotImplemented

    def __add__(self, other):
        return Add(self, self.convert(other))

    __radd__ = __add__

class AExpr(Expr):

    pass

def Number(n):
    return MyExpr(NUMBER, n)
def Symbol(s):
    return MyExpr(SYMBOL, s)
def Add(x, y):
    d = {}
    if x==y:
        d[x] = 2
    else:
        d[x] = 1
        d[y] = 1
    return MyExpr(TERMS, d)

def test_equality_number():
    assert Number(1)==Number(1)
    assert Number(1)==Number(1L)
    assert Number(1)==1
    assert Number(1)==1.0
    assert Number(1.0)==1
    assert Number(1)==1.0+0j
    assert Number(1)==1L
    assert 1==Number(1)
    assert 1L==Number(1)
    assert 1==Number(1L)
    
    assert Number(1)!=Number(2)
    assert Number(1)!=Number(2L)
    assert Number(1)!=2
    assert Number(1)!=2L
    assert 2!=Number(1)

def test_equality_symbol():
    assert Symbol('x')==Symbol('x')
    assert Symbol('x')=='x'
    assert 'x'==Symbol('x')

    assert Symbol('x')!=Symbol('y')
    assert Symbol('x')!='y'
    assert 'x'!=Symbol('y')

    assert MyExpr(SYMBOL, 'x')==AExpr(SYMBOL, 'x')

def test_pos():
    x, y, z = map(Symbol,'xyz')
    assert MyExpr(POS,x)==x
    assert hash(MyExpr(POS,x))==hash(x)
    
def test_neg():
    x, y, z = map(Symbol,'xyz')
    z, o, t = map(Number, [0,1,2])
    assert MyExpr(NEG, t)==-2, MyExpr(NEG, t)
    assert hash(MyExpr(NEG, t))==hash(-2), `hash(MyExpr(NEG, t)),hash(-2)`
    
def test_equality_add():
    x, y, z = map(Symbol,'xyz')
    assert x + y == y + x
    assert x + x == x + x
    assert x + y == (TERMS, {x:1, y:1})

    assert x + y != y + z
    assert x + y != x
    assert x + y != 'x'
    assert x + y != 1

    assert MyExpr(ADD,[])==0,`MyExpr(ADD,[]).as_lowlevel()`
    assert MyExpr(ADD,[x])==x
    assert (not not MyExpr(ADD,[])) == False

    assert hash(MyExpr(ADD,[]))==hash(0)
    assert hash(MyExpr(ADD,[x]))==hash(x)

    x1 = AExpr(SYMBOL, 'x')
    y1 = AExpr(SYMBOL, 'y')

    assert AExpr(ADD,[x1])==MyExpr(ADD,[x])
    assert AExpr(ADD,[x1,y1])==MyExpr(ADD,[x,y])
    assert not (AExpr(ADD,[x1,y1])<MyExpr(ADD,[x,y]))
    assert not (AExpr(ADD,[x1,y1])>MyExpr(ADD,[x,y]))

    assert (AExpr(ADD,[x1,y1])<=MyExpr(ADD,[x,y]))
    assert (AExpr(ADD,[x1,y1])>=MyExpr(ADD,[x,y]))

def test_equality_mul():
    x, y, z = map(Symbol,'xyz')
    assert (not not MyExpr(MUL,[])) == True
    assert MyExpr(MUL,[])==1
    assert MyExpr(MUL,[x])==x
    assert hash(MyExpr(MUL,[]))==hash(1)
    assert hash(MyExpr(MUL,[x]))==hash(x)

def test_equality_term_coeff():
    x, y, z = map(Symbol,'xyz')
    z, o, t = map(Number, [0,1,2])
    assert MyExpr(TERM_COEFF, (x, 0))==0
    assert MyExpr(TERM_COEFF, (x, 1))==x
    assert MyExpr(TERM_COEFF, (o, 2))==2
    assert MyExpr(TERM_COEFF, (x, -1))==MyExpr(NEG, x)
    assert hash(MyExpr(TERM_COEFF, (x, 0)))==hash(0)
    assert hash(MyExpr(TERM_COEFF, (x, 1)))==hash(x)
    assert hash(MyExpr(TERM_COEFF, (o, 2)))==hash(2)
    assert hash(MyExpr(TERM_COEFF, (x, -1)))==hash(MyExpr(NEG, x))

    
def test_equality_term_coeff_dict():
    x, y, z = map(Symbol,'xyz')
    z, o, t = map(Number, [0,1,2])
    assert MyExpr(TERM_COEFF_DICT, {})==0
    assert MyExpr(TERM_COEFF_DICT, {x:0})==0
    assert MyExpr(TERM_COEFF_DICT, {x:1})==x
    assert MyExpr(TERM_COEFF_DICT, {x:2})==MyExpr(TERM_COEFF, (x, 2))
    assert MyExpr(TERM_COEFF_DICT, {o:2})==2

    assert hash(MyExpr(TERM_COEFF_DICT, {}))==hash(0)
    assert hash(MyExpr(TERM_COEFF_DICT, {x:0}))==hash(0)
    assert hash(MyExpr(TERM_COEFF_DICT, {x:1}))==hash(x)
    assert hash(MyExpr(TERM_COEFF_DICT, {x:2}))==hash(MyExpr(TERM_COEFF, (x, 2)))
    assert hash(MyExpr(TERM_COEFF_DICT, {o:2}))==hash(2)

def test_equality_pow():
    x, y, z = map(Symbol,'xyz')
    z, o = map(Number, [0,1])
    assert MyExpr(POW, (x, 0))==1
    assert MyExpr(POW, (x, 1))==x
    assert MyExpr(POW, (o, x))==1

    assert hash(MyExpr(POW, (x, 0)))==hash(1)
    assert hash(MyExpr(POW, (x, 1)))==hash(x)
    assert hash(MyExpr(POW, (o, x)))==hash(1)

def test_equality_base_exp_dict():
    x, y, z = map(Symbol,'xyz')
    z, o, t = map(Number, [0,1,2])
    assert MyExpr(BASE_EXP_DICT, {})==1
    assert MyExpr(BASE_EXP_DICT, {x:0})==1
    assert MyExpr(BASE_EXP_DICT, {x:1})==x
    assert MyExpr(BASE_EXP_DICT, {x:2})==MyExpr(POW, (x, 2))
    assert MyExpr(BASE_EXP_DICT, {o:2})==1

    assert hash(MyExpr(BASE_EXP_DICT, {}))==hash(1)
    assert hash(MyExpr(BASE_EXP_DICT, {x:0}))==hash(1)
    assert hash(MyExpr(BASE_EXP_DICT, {x:1}))==hash(x)
    assert hash(MyExpr(BASE_EXP_DICT, {x:2}))==hash(MyExpr(POW, (x, 2)))
    assert hash(MyExpr(BASE_EXP_DICT, {o:2}))==hash(1)

def test_equality_exp_coeff_dict():
    x, y, z = map(Symbol,'xyz')
    assert MyExpr(EXP_COEFF_DICT, Pair((x,), {}))==0
    assert MyExpr(EXP_COEFF_DICT, Pair((x,), {IntegerList(0):2}))==2
    assert MyExpr(EXP_COEFF_DICT, Pair((x,), {IntegerList(1):1}))==x
    assert MyExpr(EXP_COEFF_DICT, Pair((x,), {IntegerList(1):2}))==MyExpr(TERM_COEFF, (x,2))
    assert MyExpr(EXP_COEFF_DICT, Pair((x,), {IntegerList(2):1}))==MyExpr(POW, (x,2))
    
def test_hash_number():
    assert hash(Number(1))==hash(1)
    assert hash(Number(-1))==hash(-1)
    assert hash(Number(1212424))==hash(1212424)
    assert hash(Number(-1212424))==hash(-1212424)

def test_hash_symbol():
    assert hash(Symbol('x'))==hash('x'),`hash(Symbol('x')),hash('x')`
    assert hash(Symbol('y'))==hash('y')

def test_hash_dict_data():
    x, y, z = map(Symbol,'xyz')
    assert hash(x + y) == hash((TERMS, frozenset([(x,1),(y,1)])))

def test_hash_list_data():
    l = map(MyExpr,[1,2,3])
    e1 = MyExpr(MUL, l)
    assert e1.is_writable
    e2 = MyExpr(MUL, tuple(l))
    assert hash(e1)==hash(e2)
    assert not e1.is_writable

def test_is_writable():
    n, m = map(MyExpr, [1,2])
    assert MyExpr(MUL, [n,m]).is_writable
    assert not MyExpr(MUL, (n,m)).is_writable
    



import pymaclab as pm
import pymaclab.modfiles.models as models

# Define the ssidic of initial guesses or starting values
alpha = 0.3
theta = 0.85
gamma = 0.25
delta_k = 0.025
betta = 0.99
B     = 0.65
inf_bar = 1.012
ssidic = {}
ssidic['k_bar']    = 30.0
ssidic['n_bar']    = 0.33
ssidic['inf_bar']  = inf_bar
ssidic['r_bar']    = 1/betta
ssidic['y_bar']    = ssidic['k_bar']**alpha*ssidic['n_bar']**(1-alpha)
ssidic['c_bar']    = ssidic['y_bar'] - delta_k*ssidic['k_bar']
ssidic['m_bar']    = 5.0
# Define marginal cost
rk = alpha*(ssidic['n_bar']/ssidic['k_bar'])**(1-alpha)
rn = (1-alpha)*(ssidic['k_bar']/ssidic['n_bar'])**alpha
mc = (1/(1-alpha))**(1-alpha)*(1/alpha)**alpha*(rk)**alpha*(rn)
ssidic['aa_bar']   = inf_bar*mc*ssidic['y_bar']/(1-(1-gamma)*betta*ssidic['inf_bar']**((2-theta)/(1-theta)))
ssidic['bb_bar']   = ssidic['y_bar']/(1-(1-gamma)*betta*ssidic['inf_bar']**(1/(1-theta)))
ssidic['infr_bar'] = (1/theta)*(ssidic['aa_bar']/ssidic['bb_bar'])
ssidic['i_bar']    = ssidic['r_bar']*inf_bar
ssidic['lam_bar']  = 1.0/ssidic['c_bar']
ssidic['w_bar'] = (1/theta)*rn

# Instantiate a new DSGE model instance like so
foc_li = [0,1,2,3,4,5,6,7,8,9,10,11,12]
nkm1 = pm.newMOD(models.nkm,use_focs=foc_li,ssidic=ssidic,initlev=0) 


#
# Author: Fredrik Johansson
# Created: January 2008
""" Provides UnivariatePolynomial class.
"""
__docformat__ = "restructuredtext"
__all__ = ['UnivariatePolynomial', 'poly']

from ..utils import ADD, POW, MUL, NUMBER, SYMBOL, DENSE_POLY
from ..basealgebra.verbatim import Verbatim
from ..basealgebra import Algebra
from ..arithmetic.numbers import div

def poly(coefs=[], symbol='x', coerce_input=True):
    coefs = coefs or [0]
    if coerce_input:
        coefs = map(UnivariatePolynomial.convert_coef, coefs)
    return UnivariatePolynomial(DENSE_POLY, (symbol, normalized(coefs)))

def normalized(coefs):
    while coefs and not coefs[-1]:
        coefs = coefs[:-1]
    return tuple(coefs)

class UnivariatePolynomial(Algebra):
    """ Represents (dense) univariate polynomial.
    """

    @classmethod
    def convert(cls, data, typeerror=True):
        if isinstance(data, (list, tuple)):
            coefs = data or [0]
            return cls(DENSE_POLY, ('x', normalized(coefs)))
        return super(Algebra, cls).convert(data, typeerror=True)

    def __nonzero__(self):
        return not not self.data[1]

    @property
    def degree(self):
        return len(self.data[1])-1

    @classmethod
    def convert_coef(cls, x):
        # permit anything by default
        return x

    def as_verbatim(self):
        if self.degree == -1:
            return Verbatim(NUMBER, 0)
        t = []
        head, (symbol, data) = self.pair
        x = Verbatim(SYMBOL, symbol)
        for exp, coef in enumerate(data):
            if coef == 0:
                continue
            scoef = Verbatim(NUMBER, coef)
            if exp == 0:
                monomial = scoef
            else:
                monomial = x
                if exp != 1: monomial = monomial ** Verbatim(NUMBER, exp)
                if coef != 1: monomial = scoef * monomial
            t.append(monomial)
        if t:
            return Verbatim(ADD, tuple(t))
        return Verbatim(NUMBER, 0)

    @property
    def tree(self):
        return self.as_verbatim().tree

    def __call__(self, x):
        head, (symbol, data) = self.pair
        cls = type(self)
        if isinstance(x, UnivariatePolynomial):
            p = cls(x.head, (symbol,()))
            xp = cls(x.head, (symbol,(1,)))
            for i, c in enumerate(data):
                p += c * xp
                xp *= x
            return p
        else:
            y = cls.convert_coef(0)
            for c in reversed(data):
                y = y*x+c
            return y

    def __neg__(self):
        head, (symbol, data) = self.pair
        return type(self)(head, (symbol, tuple([-c for c in data])))

    def __add__(self, other):
        head, (symbol, data) = self.pair
        cls = type(self)
        if not isinstance(other, UnivariatePolynomial):
            other = cls(head, (symbol,(other,)))
        ohead, (osymbol, odata) = other.pair
        if symbol != osymbol:
            return NotImplemented
        coefs = [a+b for (a, b) in zip(data, odata)]
        if self.degree >= other.degree:
            coefs += data[other.degree+1:]
        else:
            coefs += odata[self.degree+1:]
        return cls(head, (symbol, normalized(coefs)))

    __radd__ = __add__

    def __sub__(self, other):
        return self + (-other)

    def __rsub__(self, other):
        return (-self) + other

    def __mul__(self, other):
        head, (symbol, data) = self.pair
        cls = type(self)
        if not isinstance(other, UnivariatePolynomial):
            try:
                other = self.convert_coef(other)
                coefs = [other*c for c in data]
                return cls(head, (symbol, normalized(coefs)))
            except:
                pass
            return NotImplemented
        ohead, (osymbol, odata) = other.pair
        if osymbol != osymbol:
            return NotImplemented
        p = [self.convert_coef(0)] * (len(data)+len(odata))
        for i, c in enumerate(data):
            for j, d in enumerate(odata):
                p[i+j] += c*d
        return cls(head, (symbol, normalized(p)))

    __rmul__ = __mul__

    def __pow__(self, n):
        # TODO: Miller's algorithm
        assert isinstance(n, int) and n >= 0
        head, (symbol, data) = self.pair
        if n == 0: return type(self)(head, (symbol, (1,)))
        if n == 1: return self
        if n & 1 == 0: return (self*self)**(n//2)
        if n & 1 == 1: return self*((self*self)**(n//2))
        return NotImplemented

    def __divmod__(self, other):
        cls = type(self)
        head, (symbol, data) = self.pair
        if isinstance(other, UnivariatePolynomial):
            ohead, (osymbol, odata) = other.pair
            if symbol != osymbol:
                return NotImplemented
            if other.degree < 0:
                raise ZeroDivisionError, "polynomial division"
            n = self.degree
            nv = other.degree
            u = data
            v = odata
            r = list(u)
            q = [0] * (len(r)+1)
            for k in range(n-nv, -1, -1):
                q[k] = div(r[nv+k], v[nv])
                for j in range(nv+k-1, k-1, -1):
                    r[j] -= q[k]*v[j-k]
            for j in range(nv, n+1, 1):
                r[j] = 0
            return cls(head, (symbol, normalized(q))), cls(head, (symbol, normalized(r)))
        else:
            if other == 0:
                raise ZeroDivisionError, "polynomial division"
            rec = div(1, other)
            q = cls(head, (symbol, tuple([c*rec for c in data])))
            r = cls(head, (symbol, ()))
            return q, r

    def __div__(self, other):
        return divmod(self, other)[0]

    __truediv__ = __div__

    def __mod__(self, other):
        return divmod(self, other)[1]

    def diff(self):
        head, (symbol, data) = self.pair
        return type(self)(head, (symbol, tuple([c*(k+1) for k, c in enumerate(data[1:])])))




""" Provides matrix utility functions.
"""
#
# Author: Pearu Peterson
# Created: March 2008
#

__docformat__ = "restructuredtext"
__all__ = ['eye', 'concatenate', 'jacobian']

from ..utils import MATRIX, MATRIX_DICT
from .algebra import MatrixDict, Matrix, MatrixBase

def jacobian(expr_list, var_list):
    """ Return a jacobian matrix of functions in expr_list with
    respect to variables in var_list.
    """
    m, n = len(expr_list), len(var_list)
    jac = Matrix(m, n)
    for i, e in enumerate(expr_list):
        for j, v in enumerate(var_list):
            jac[i,j] = e.diff(v)
    return jac

def eye(m, n=None, k=0):
    """ Return n x m matrix where the k-th diagonal is all ones,
    everything else is zeros.
    """
    if n is None:
        n = m
    d = {}
    if k<0:
        for i in range(min(m,n)+k):
            d[i-k,i] = 1
    else:
        for i in range(min(m,n)-k):
            d[i,i+k] = 1
    return MatrixDict(MATRIX(m, n, MATRIX_DICT), d)

def concatenate(*args, **kws):
    """ Join matrices together.

    concatenate(m1, m2, ..) - join matrices m1, m2, .. together along rows
    concatenate(m1, m2, .., axis=1) - join matrices m1, m2, .. together along columns
    concatenate(m1, m2, .., diagonal=True) - join matrices m1, m2, .. together along diagonal
    """
    assert args,`args`
    assert set(kws).issubset(['axis','diagonal']) and len(kws)<=1,`kws`
    axis = kws.get('axis',0)
    diagonal = kws.get('diagonal',False)
    d = {}
    rows, cols = 0, 0
    if axis:
        for a in args:
            head, data = Matrix(a)[:].pair
            assert not (head.is_transpose or head.is_diagonal)
            m, n = head.shape
            for (i,j),x in data.items():
                d[rows + i,j] = x
            rows += m
            cols = max(cols, n)
    elif diagonal:
        for a in args:
            head, data = Matrix(a)[:].pair
            assert not (head.is_transpose or head.is_diagonal)
            m, n = head.shape
            for (i,j),x in data.items():
                d[rows + i, cols + j] = x
            rows += m
            cols += n
    else:
        for a in args:
            head, data = Matrix(a)[:].pair
            assert not (head.is_transpose or head.is_diagonal)
            m, n = head.shape
            for (i,j),x in data.items():
                d[i,cols+j] = x
            cols += n
            rows = max(rows, m)
    return MatrixDict(MATRIX(rows, cols, MATRIX_DICT), d)



#!/usr/bin/env python

CLASSIFIERS = """\
Development Status :: 3 - Alpha
Intended Audience :: Science/Research
License :: OSI Approved
Programming Language :: Python
Topic :: Scientific/Engineering
Topic :: Software Development
Operating System :: Microsoft :: Windows
Operating System :: POSIX
Operating System :: Unix
Operating System :: MacOS
"""

import os
if os.path.exists('MANIFEST'): os.remove('MANIFEST')

from distutils.core import Extension, Command
from distutils.command.build_py import build_py as _build_py

expr_ext = Extension('sympycore.expr_ext',
                     sources = [os.path.join('src','expr_ext.c')],
                     )

combinatorics_ext = Extension('sympycore.arithmetic.combinatorics',
                     sources = [os.path.join('sympycore','arithmetic','combinatorics.c')],
                     )

extensions = [expr_ext,
              #combinatorics_ext
              ]

packages = ['sympycore',
            'sympycore.algebras',
            'sympycore.arithmetic',
            'sympycore.arithmetic.mpmath',
            'sympycore.arithmetic.mpmath.calculus',
            'sympycore.arithmetic.mpmath.functions',
            'sympycore.arithmetic.mpmath.libmp',
            'sympycore.arithmetic.mpmath.matrices',
            'sympycore.basealgebra',
            'sympycore.calculus',
            'sympycore.calculus.functions',
            'sympycore.functions',
            'sympycore.heads',
            'sympycore.logic',
            'sympycore.matrices',
            'sympycore.polynomials',
            'sympycore.physics',
            'sympycore.physics.sysbio',
            'sympycore.ring',
            'sympycore.sets',
            ]

packages += [p+'.tests' for p in packages \
             if os.path.exists(os.path.join(p.replace('.', os.sep), 'tests'))]

class tester(Command):
    description = "run sympycore tests"
    user_options = [('nose-args=', 'n', 'arguments to nose command'),
                    ('with-coverage', 'c', 'use nose --with-coverage flag'),
                    ('cover-package=', None, 'use nose --cover-package flag'),
                    ('detailed-errors', 'd', 'use nose --detailed-errors flag'),
                    ('nocapture', 's', 'use nose --nocapture flag'),
                    ('nose-verbose', 'v', 'use nose --verbose flag'),
                    ('match=', 'm', 'use nose --match flag'),
                    ('profile', 'p', 'use nose --profile flag'),
                    ('with-doctest', None, 'use nose --with-doctest flag'),
                    ('stop', 'x', 'use nose --stop flag')
                    ]
    def initialize_options(self):
        self.nose_args = None
        self.with_coverage = None
        self.cover_package = None
        self.detailed_errors = None
        self.nocapture = None
        self.nose_verbose = None
        self.match = None
        self.profile = None
        self.with_doctest = None
        self.stop = None
        return
    def finalize_options (self):
        if self.nose_args is None:
            self.nose_args = ''
        if self.with_coverage:
            self.nose_args += ' --with-coverage'
        if self.cover_package:
            if not self.with_coverage:
                self.nose_args += ' --with-coverage'
            self.nose_args += ' --cover-package=%s' % self.cover_package
        elif self.with_coverage:
            self.nose_args += ' --cover-package=sympycore'
        if self.detailed_errors:
            self.nose_args += ' --detailed-errors'
        if self.nocapture:
            self.nose_args += ' --nocapture'
        if self.nose_verbose:
            self.nose_args += ' --verbose'
        if self.match:
            self.nose_args += ' --match=%r' % (self.match)
        if self.profile:
            self.nose_args += ' --with-profile'
        if self.with_doctest:
            self.nose_args += ' --with-doctest'
        if self.stop:
            self.nose_args += ' --stop'
        return
    def run(self):
        import sympycore
        sympycore.test(nose_args=self.nose_args)

class build_py(_build_py):

    def find_data_files (self, package, src_dir):

        files = _build_py.find_data_files(self, package, src_dir)

        if package=='sympycore':
            revision = self._get_svn_revision(src_dir)
            if revision is not None:
                target = os.path.join(src_dir, '__svn_version__.py')
                print 'Creating ', target
                f = open(target,'w')
                f.write('version = %r\n' % (str(revision)))
                f.close()
                import atexit
                def rm_file(f=target):
                    try: os.remove(f); print 'Removed ',f
                    except OSError: pass
                    try: os.remove(f+'c'); print 'Removed ',f+'c'
                    except OSError: pass
                atexit.register(rm_file)
                files.append(target)

        if package=='sympycore.arithmetic.mpmath':
            f = os.path.join(src_dir, 'REVISION')
            if os.path.isfile(f):
                files.append(f)

        return files

    def _get_svn_revision(self, path):
        """Return path's SVN revision number.
        """
        import os, sys, re
        revision = None
        m = None
        try:
            sin, sout = os.popen4('svnversion')
            m = re.match(r'(?P<revision>\d+)', sout.read())
        except:
            pass
        if m:
            revision = int(m.group('revision'))
            return revision
        if sys.platform=='win32' and os.environ.get('SVN_ASP_DOT_NET_HACK',None):
            entries = os.path.join(path,'_svn','entries')
        else:
            entries = os.path.join(path,'.svn','entries')
        if os.path.isfile(entries):
            f = open(entries)
            fstr = f.read()
            f.close()
            if fstr[:5] == '<?xml':  # pre 1.4
                m = re.search(r'revision="(?P<revision>\d+)"',fstr)
                if m:
                    revision = int(m.group('revision'))
            else:  # non-xml entries file --- check to be sure that
                m = re.search(r'dir[\n\r]+(?P<revision>\d+)', fstr)
                if m:
                    revision = int(m.group('revision'))
        return revision

        
from distutils.core import setup
setup(name='sympycore',
      version='0.2-svn',
      author = 'Pearu Peterson, Fredrik Johansson',
      author_email = 'sympycore@googlegroups.com',
      license = 'http://sympycore.googlecode.com/svn/trunk/LICENSE',
      url = 'http://sympycore.googlecode.com',
      download_url = 'http://code.google.com/p/sympycore/downloads/',
      classifiers=filter(None, CLASSIFIERS.split('\n')),
      description = 'SympyCore: an efficient pure Python Computer Algebra System',
      long_description = '''\
SympyCore project provides a pure Python package sympycore for
representing symbolic expressions using efficient data structures as
well as methods to manipulate them. Sympycore uses a clear algebra
oriented design that can be easily extended.
''',
      platforms = ["All"],
      packages = packages,
      ext_modules = extensions,
      package_dir = {'sympycore': 'sympycore'},
      cmdclass=dict(test=tester, build_py=build_py)
      )




# Import the pymaclab module into its namespace, also import os module
import pymaclab as pm
from pymaclab.modfiles import models

def test7():
    # Define the ssidic of initial guesses or starting values
    alpha      = 0.36
    chi        = 0.819
    xi         = 1.0/4.3
    g          = 0.005
    tau        = 5.0
    betta      = 1.014
    bettaa     = betta*(1+g)**(-tau)
    delta      = 0.025
    rho        = 0.95
    a1         = (g+delta)**(1.0/xi)
    a2         = (g+delta)/(1.0-xi)
    z_bar      = 1.0
    sigma_eps  = 0.01
    
    ssidic = {}
    ssidic['k_bar']     = 36.0                                                  #1
    ssidic['inv_bar']   = 1.0                                                   #2
    ssidic['c_bar']     = 2.5                                                   #3
    ssidic['mu_bar']    = 8.0                                                   #4

    # Pick first FOC equations to leave out exogenous law of motion
    focs = range(4)

    # Instantiate a new DSGE model instance like so
    asset = pm.newMOD(models.stable.jermann98_ext,mesg=True,use_focs=focs,ssidic=ssidic)
    
    return asset




# Author: Pearu Peterson
# Created: April 2011

__all__ = ['load_stoic_from_sbml',
           'load_stoic_from_text',
           ]

import re
import os
from collections import defaultdict

from .utils import obj2num


def load_stoic_from_sbml(file_name, split_bidirectional_fluxes=False):
    """ Return stoichiometry information of a network described in a SBML file.

    Parameters
    ----------
    file_name : str
      Path to SMBL file.

    split_bidirectional_fluxes : bool
      When True the bidirectional fluxes are split into two unidirectional fluxes.
      For example, the system ``A<=>B`` is treated as ``A=>B and B=>A``.

    Returns
    -------
    matrix : dict
      A stoichiometry matrix defined as mapping {(species, reaction): stoichiometry}.
    species : list
      A list of species names.
    reactions : list
      A list of reaction names.
    species_info : dict
    reactions_info : dict
    """
    from lxml import etree
    for ext in ['', '.xml', '.xml.gz', '.gz']:
        if os.path.isfile(file_name+ext):
            file_name = file_name+ext
            break
    tree = etree.parse(file_name)
    root = tree.getroot()
    assert root.tag.endswith ('sbml'), `root.tag`
    version = int(root.attrib['version'])
    level = int(root.attrib['level'])
    if level in [2,3]:
        default_stoichiometry = '1'
    else:
        default_stoichiometry = None
    compartments = {}
    species = []
    modifiers = []
    species_all = []
    reactions = []
    species_reactions = defaultdict (lambda:[])
    reactions_species = defaultdict (lambda:[])
    reactions_info = defaultdict(lambda:dict(modifiers=[],reactants=[],products=[],
                                            boundary_specie_stoichiometry={},annotation=[],
                                            compartments = set()))
    species_info = defaultdict(lambda:dict())
    matrix = {}
    for model in root:
        for item in model:
            if item.tag.endswith('listOfCompartments'):
                for compartment in item:
                    compartments[compartment.attrib['id']] = compartment.attrib
            elif item.tag.endswith('listOfSpecies'):
                for specie in item:
                    species_all.append(specie.attrib['id'])
                    species_info[specie.attrib['id']]['compartment'] = specie.attrib['compartment']
                    species_info[specie.attrib['id']]['name'] = specie.attrib.get('name', specie.attrib['id'])
            elif item.tag.endswith('listOfReactions'):
                for reaction in item:

                    reversible =eval(reaction.attrib.get('reversible', 'False').title())
                    reaction_id = reaction.attrib['id']
                    name = reaction.attrib.get('name', reaction_id)
                    assert reaction_id not in reactions,`reaction_id`
                    reactions.append(reaction_id)
                    reaction_index = len(reactions)-1
                    reactions_info[reaction_id]['name'] = name

                    if split_bidirectional_fluxes and reversible:
                        reaction_id2 = '%s_r' % (reaction_id)
                        assert reaction_id2 not in reactions,`reaction_id2`
                        reactions.append(reaction_id2)
                        reaction_index2 = len(reactions)-1
                        reactions_info[reaction_id2]['name'] = name+'_r'
                        reactions_info[reaction_id]['reversible'] = False
                        reactions_info[reaction_id2]['reversible'] = False
                    else:
                        reaction_id2 = reaction_index2 = None
                        reactions_info[reaction_id]['reversible'] = reversible

                    for part in reaction:
                        if part.tag.endswith ('listOfReactants'):
                            for reactant in part:
                                assert reactant.tag.endswith('speciesReference'), `reactant.tag`
                                specie_id = reactant.attrib['species']
                                stoichiometry = -obj2num(reactant.attrib.get('stoichiometry', default_stoichiometry))
                                reactions_info[reaction_id]['reactants'].append(specie_id)
                                try:
                                    specie_index = species.index(specie_id)
                                except ValueError:
                                    species.append(specie_id)
                                    specie_index = len(species)-1
                                assert stoichiometry,`stoichiometry`
                                matrix[specie_index, reaction_index] = stoichiometry
                                species_reactions[specie_index].append(reaction_index)
                                reactions_species[reaction_index].append(specie_index)                                
                                reactions_info[reaction_id]['compartments'].add(species_info[specie_id]['compartment'])
                                if reaction_index2 is not None:
                                    reactions_info[reaction_id2]['reactants'].append(specie_id)
                                    matrix[specie_index, reaction_index2] = -stoichiometry
                                    species_reactions[specie_index].append(reaction_index2)
                                    reactions_species[reaction_index2].append(specie_index)                                
                                    reactions_info[reaction_id2]['compartments'].add(species_info[specie_id]['compartment'])
                        elif part.tag.endswith ('listOfProducts'):
                            for product in part:
                                assert product.tag.endswith('speciesReference'), `product.tag`
                                specie_id = product.attrib['species']
                                stoichiometry = obj2num(product.attrib.get('stoichiometry', default_stoichiometry))
                                reactions_info[reaction_id]['products'].append(specie_id)
                                try:
                                    specie_index = species.index(specie_id)
                                except ValueError:
                                    species.append(specie_id)
                                    specie_index = len(species)-1
                                assert stoichiometry,`stoichiometry`
                                matrix[specie_index, reaction_index] = stoichiometry
                                species_reactions[specie_index].append(reaction_index)
                                reactions_species[reaction_index].append(specie_index)
                                reactions_info[reaction_id]['compartments'].add(species_info[specie_id]['compartment'])
                                if reaction_index2 is not None:
                                    reactions_info[reaction_id2]['products'].append(specie_id)
                                    matrix[specie_index, reaction_index2] = -stoichiometry
                                    species_reactions[specie_index].append(reaction_index2)
                                    reactions_species[reaction_index2].append(specie_index)
                                    reactions_info[reaction_id2]['compartments'].add(species_info[specie_id]['compartment'])
                        elif part.tag.endswith ('listOfModifiers'):
                            for modifier in part:
                                assert modifier.tag.endswith('modifierSpeciesReference'), `modifier.tag`
                                specie_id = product.attrib['species']
                                reactions_info[reaction_id]['modifiers'].append(specie_id)
                                reactions_info[reaction_id]['compartments'].add(species_info[specie_id]['compartment'])
                            continue
                        elif part.tag.endswith ('annotation'):
                            reactions_info[reaction_id]['annotation'].append(part.text)
                            continue
                        elif re.match(r'.*(kineticLaw|notes)\Z', part.tag):                            
                            continue
                        else:
                            print 'get_stoichiometry:warning:unprocessed reaction element: %r' % (part.tag)
                            continue


            elif re.match (r'.*(annotation|notes|listOfSpeciesTypes|listOfUnitDefinitions)\Z', item.tag):
                pass
            else:
                print 'get_stoichiometry:warning:unprocessed model element: %r' % (item.tag)

    return matrix, species, reactions, species_info, reactions_info
    

def load_stoic_from_text(text, split_bidirectional_fluxes=False):
    """ Parse stoichiometry matrix from a string.

    Parameters
    ----------
    text : str
      A multiline string where each line contains a chemical reaction
      description. The description must be given in the following
      form: ``<sum of reactants> (=> | <=) <sum of producats>``. For example,
      ``A + 2*B => C``. Lines starting with ``#`` are ignored.

    split_bidirectional_fluxes : bool
      When True the bidirectional fluxes are split into two unidirectional fluxes.
      For example, the system ``A<=>B`` is treated as ``A=>B and B=>A``.

    Returns
    -------
    matrix_data : dict
      A dictionary representing a stoichiometry matrix.

    species : list
      A list of row names.

    reactions : list
      A list of column names.

    species_info : dict
    reactions_info : dict
    """
    #TODO: fill up species_info and reactions_info dictionaries

    def _split_sum (line):
        for part in line.split('+'):
            part = part.strip()
            coeff = ''
            while part and part[0].isdigit():
                coeff += part[0]
                part = part[1:].lstrip()
            if not coeff:
                coeff = '1'
            if not part:
                continue
            yield part, eval (coeff)

    matrix = {}
    reactions = []
    species = []
    reactions_info = defaultdict(lambda:dict(modifiers=[],reactants=[],products=[],
                                            boundary_specie_stoichiometry={},annotation=[],
                                            compartments = set()))
    species_info = defaultdict(lambda:dict())
    for line in text.splitlines():
        line = line.strip()
        if not line or line.startswith ('#'): continue
        if ':' in line:
            reaction_name, line = line.split (':',1)
            reaction_name = reaction_name.strip()
            line = line.strip()
        else:
            reaction_name = None
        if not line or line.startswith ('#'): continue

        reaction_string = line

        reversible = False
        left, right = line.split ('=')
        direction = '='
        if right.startswith('>'):
            right = right[1:].strip()
            direction = '>'
            if left.endswith ('<'):
                left = left[:-1].strip()
                reversible = True
        elif left.endswith ('>'):
            left = left[:-1].strip()
            direction = '>'
        elif left.endswith ('<'):
            left = left[:-1].strip()
            direction = '<'
        elif right.startswith ('<'):
            right = right[1:].strip()
            direction = '<'

        left_specie_coeff = list(_split_sum(left))
        right_specie_coeff = list(_split_sum(right))
        left_specie_names = [n for n,c in left_specie_coeff if n]
        right_specie_names = [n for n,c in right_specie_coeff if n]

        fname = ['R']
        rname = ['R']
        name0 = ''.join(left_specie_names)
        name1 = ''.join(right_specie_names)
        if name0:
            rname.append (name0)
        if name1:
            fname.append (name1)
            rname.append (name1)
        if name0:
            fname.append (name0)

        if direction=='<':
            if not reaction_name:
                reaction_name = '_'.join(fname)
                reaction_name2 = '_'.join(rname)
            else:
                reaction_name2 = 'r'+reaction_name
                if split_bidirectional_fluxes:
                    reaction_name = 'f'+reaction_name
        else:
            if not reaction_name:
                reaction_name2 = '_'.join(fname)
                reaction_name = '_'.join(rname)
            else:
                reaction_name2 = 'r'+reaction_name
                if split_bidirectional_fluxes:
                    reaction_name = 'f'+reaction_name

        reactions.append (reaction_name)
        reaction_index = reactions.index (reaction_name)
        if split_bidirectional_fluxes and reversible:
            reactions.append (reaction_name2)
            reaction_index2 = reactions.index (reaction_name2)
        else:
            reaction_index2 = None

        def matrix_add (i,j,c):
            v = matrix.get ((i,j))
            if v is None:
                matrix[i, j] = c
            else:
                matrix[i, j] = v + c

        for specie, coeff in left_specie_coeff:
            if specie not in species:
                species.append (specie)
            specie_index = species.index (specie)
            if direction=='<':
                if reaction_index2 is not None:
                    matrix_add(specie_index, reaction_index2, -coeff)
                matrix_add(specie_index, reaction_index, coeff)
            else:
                if reaction_index2 is not None:
                    matrix_add(specie_index, reaction_index2, coeff)
                matrix_add(specie_index, reaction_index, -coeff)
            
        for specie, coeff in right_specie_coeff:
            if specie not in species:
                species.append (specie)
            specie_index = species.index (specie)
            if direction=='<':
                if reaction_index2 is not None:
                    matrix_add(specie_index, reaction_index2, coeff)
                matrix_add(specie_index, reaction_index, -coeff)
            else:
                if reaction_index2 is not None:
                    matrix_add(specie_index, reaction_index2, -coeff)
                matrix_add(specie_index, reaction_index, coeff)

        if split_bidirectional_fluxes:
            reactions_info[reaction_name]['reversible'] = False
            reactions_info[reaction_name]['reactants'] = left_specie_names
            reactions_info[reaction_name]['products'] = right_specie_names
            reactions_info[reaction_name]['forward'] = reaction_name
            reactions_info[reaction_name]['reverse'] = None

            if reversible:
                reactions_info[reaction_name2]['reversible'] = False
                reactions_info[reaction_name2]['reactants'] = right_specie_names
                reactions_info[reaction_name2]['products'] = left_specie_names
                reactions_info[reaction_name2]['forward'] = reaction_name2
                reactions_info[reaction_name2]['reverse'] = None
            # TODO: set  reactions_info[reaction_name]['name']
        else:
            reactions_info[reaction_name]['reversible'] = reversible
            reactions_info[reaction_name]['reactants'] = left_specie_names
            reactions_info[reaction_name]['products'] = right_specie_names
            if reversible:
                reactions_info[reaction_name]['forward'] = 'f'+reaction_name
                reactions_info[reaction_name]['reverse'] = 'r'+reaction_name
            else:
                reactions_info[reaction_name]['forward'] = 'f'+reaction_name
                reactions_info[reaction_name]['reverse'] = None

            reactions_info[reaction_name]['name'] = reaction_string

    return matrix, species, reactions, species_info, reactions_info



'''
.. module:: queued
   :platform: Linux
   :synopsis: A collection of tools required for doing intelligent and dynamic DSGE model instance updating at runtime. The version in
              this module is for the queued updater behaviour.

.. moduleauthor:: Eric M. Scheffel <eric.scheffel@nottingham.edu.cn>


'''
from copy import deepcopy
from ..solvers.steadystate import ManualSteadyState

queue = []

class Updaters_Queued(object):
    def __init__(self):
        pass

class dicwrap_queued:
    def __init__(self,other,wrapobj_str,initlev):
        self.other = other
        self.queue = other.updaters_queued.queue
        self.wrapobj_str = wrapobj_str
        wrapobj = eval('other.'+wrapobj_str.split('.')[1])
        self.wrapobj = wrapobj
        self.initlev = initlev
        if wrapobj_str == 'self.vardic':
            self.wrapdic = deepcopy(other.vardic)
        elif wrapobj_str == 'self.nlsubsdic':
            self.wrapdic = deepcopy(other.nlsubsdic)
        elif wrapobj_str == 'self.paramdic':
            self.wrapdic = deepcopy(other.paramdic)
            

    def __getattr__(self,attrname):
        return getattr(self.wrapdic,attrname)


    def __setitem__(self,key,value):
        other = self.other
        initlev = self.initlev
        wrapobj_str = self.wrapobj_str
        wrapobj = self.wrapobj
        mesg = other._mesg
        # ...and assign before test of inequality
        old_value = deepcopy(wrapobj[key])
        wrapobj[key] = value
        if self.wrapdic != wrapobj:
            if mesg:
                print "You have UPDATED in object "+wrapobj_str+"['"+key+"']:"
                print str(old_value)+' --> '+str(value)
            # Update the wrapdic to be identical with wrapobj
            self.wrapdic.update(wrapobj)
            if wrapobj_str == 'self.nlsubsdic':
                for i1,elem in enumerate(other.nlsubs_raw1):
                    other.nlsubs_raw1[i1][1] = deepcopy(wrapobj[other.nlsubs_raw1[i1][0]])
                if 'self.nlsubsdic' not in self.queue: self.queue.append('self.nlsubsdic')
            elif wrapobj_str == 'self.paramdic':
                if 'self.paramdic' not in self.queue: self.queue.append('self.paramdic')
            elif wrapobj_str == 'self.vardic':
                if 'self.vardic' not in self.queue: self.queue.append('self.vardic')           

    def __getitem__(self,key):
        return self.wrapdic[key]

    def update(self,dico):
        self.updic = dico
        other = self.other
        initlev = self.initlev
        wrapobj_str = self.wrapobj_str
        wrapobj = self.wrapobj
        # ...and update befor test of inequality
        wrapobj.update(dico)
        # Check if new keys are already present in wrapdic
        for keyo in dico.keys():
            if keyo not in self.wrapdic.keys():
                print "ERROR: You can only update existing keys, not introduce new ones."
                return
        # Check if any key's value has been updated relative to wrapdic
        if self.wrapdic != wrapobj:
            self.wrapdic.update(wrapobj)
            if wrapobj_str == 'self.nlsubsdic':
                for i1,elem in enumerate(other.nlsubs_raw1):
                    other.nlsubs_raw1[i1][1] = deepcopy(self.nlsubsdic[other.nlsubs_raw1[i1][0]])
                if 'self.nlsubsdic' not in self.queue: self.queue.append('self.nlsubsdic')
            elif wrapobj_str == 'self.paramdic':
                if 'self.paramdic' not in self.queue: self.queue.append('self.paramdic')
            elif wrapobj_str == 'self.vardic':
                if 'self.vardic' not in self.queue: self.queue.append('self.vardic')           


    def __repr__(self):
        return self.wrapdic.__repr__()
    def __str__(self):
        return self.wrapdic.__str__()

class listwrapk:
    def __init__(self,other,wrapobj,initlev):
        self.other = other
        self.wrapobj = wrapobj
        self.initlev = initlev
        self.wrapli = deepcopy(wrapobj)
        self.queue = other.updaters_queued.queue
            
    def __getattr__(self,attrname):
        return getattr(self.wrapli,attrname)

    def __setslice__(self,ind1,ind2,into):
        other = self.other
        wrapobj = self.wrapobj
        initlev = self.initlev
        lengo = len(self.wrapli)
        if ind2 >= lengo:
            print "ERROR: Assignment out of bounds of original list"
            return
        wrapobj[ind1:ind2] = into
        if self.wrapli[ind1:ind2] != wrapobj[ind1:ind2]:
            self.wrapli[ind1:ind2] = into
            # Also update the wrapobj at the top level of the deepwrapdic
            self.other.updaters_queued.vardic.all_update()
            self.queue.append('self.vardic')
    
    def __setitem__(self,ind,into):
        other = self.other
        initlev = self.initlev
        wrapobj = self.wrapobj
        lengo = len(self.wrapli)
        if ind >= lengo:
            print "ERROR: Assignment out of bounds of original list"
            return
        wrapobj[ind] = into
        if self.wrapli[ind] != wrapobj[ind]:
            self.wrapli[ind] = into
            # Also update the wrapobj at the top level of the deepwrapdic
            self.other.updaters_queued.vardic.all_update()
            if 'self.vardic' not in self.queue: self.queue.append('self.vardic')
            
    def __getitem__(self,ind):
        lengo = len(self.wrapli)
        if ind >= lengo:
            print "ERROR: Assignment out of bounds of original list"
            return
        else:
            return self.wrapli[ind]

    def __repr__(self):
        return self.wrapli.__repr__()
    def __str__(self):
        return self.wrapli.__str__()

class dicwrapk:
    def __init__(self,other,wrapobj,initlev):
        self.other = other
        self.wrapobj = wrapobj
        self.initlev = initlev
        self.wrapdic = deepcopy(wrapobj)
        self.queue = other.updaters_queued.queue

    def __getattr__(self,attrname):
        return getattr(self.wrapdic,attrname)

    def __setitem__(self,key,value):
        other = self.other
        initlev = self.initlev
        wrapobj = self.wrapobj
        wrapobj[key] = value
        # Test if the dictionary has changed relative to self.wrapdic
        if self.wrapdic != wrapobj:
            self.wrapdic[key] = deepcopy(wrapobj[key])
            # Also update the wrapobj at the top level of the deepwrapdic
            self.other.updaters_queued.vardic.all_update()
            if 'self.vardic' not in self.queue: self.queue.append('self.vardic')

    def __getitem__(self,key):
        return self.wrapdic[key]

    def update(self,dico):
        self.updic = dico
        other = self.other
        initlev = self.initlev
        wrapobj = self.wrapobj
        wrapobj.update(dico)
        # Check if new keys are already present in wrapdic
        for keyo in dico.keys():
            if keyo not in self.wrapdic.keys():
                print "ERROR: You can only update existing keys, not introduce new ones."
                return
        # Check if any key's value has been updated relative to wrapdic
        if self.wrapdic != wrapobj:
            self.wrapdic.update(wrapobj)
            # Also update the wrapobj at the top level of the deepwrapdic
            self.other.updaters_queued.vardic.all_update()
            if 'self.vardic' not in self.queue: self.queue.append('self.vardic')


    def __repr__(self):
        return self.wrapdic.__repr__()
    def __str__(self):
        return self.wrapdic.__str__()    

class dicwrap_deep_queued:
    def __init__(self,other,wrapobj_str,initlev):
        self.other = other
        self.wrapobj_str = wrapobj_str
        self.initlev = initlev
        self.queue = other.updaters_queued.queue
        # Create the wrapobj using the passed string
        self.wrapobj = eval('other.'+wrapobj_str.split('.')[1])
        if wrapobj_str == 'self.vardic':
            self.wrapdic = deepcopy(self.wrapobj)
            for keyo in self.wrapdic.keys():
                self.wrapdic[keyo] = dicwrapk(other,self.wrapdic[keyo],initlev)
                for keyo2 in self.wrapdic[keyo].keys():
                    self.wrapdic[keyo][keyo2] = dicwrapk(other,self.wrapdic[keyo][keyo2],initlev)
                    for i1,elem in enumerate(self.wrapdic[keyo][keyo2]):
                        self.wrapdic[keyo][keyo2][i1] = listwrapk(other,self.wrapdic[keyo][keyo2][i1],initlev)

    def __getattr__(self,attrname):
        return getattr(self.wrapdic,attrname)
    
    def all_update(self):
        self.wrapobj.update(self.wrapdic)
                       
    def __setitem__(self,key,value):
        other = self.other
        initlev = self.initlev
        wrapobj_str = self.wrapobj_str
        wrapobj = self.wrapobj
        wrapobj[key] = value
        # Test if the dictionary has changed relative to self.wrapdic
        if self.wrapdic != wrapobj:
            self.wrapdic[key] = value
            # Also update the wrapobj at the top level of the deepwrapdic
            self.other.updaters_queued.vardic.all_update()
            if wrapobj_str == 'self.vardic':
                if 'self.vardic' not in self.queue: self.queue.append('self.vardic')


    def __update__(self,dico):
        other = self.other
        initlev = self.initlev
        wrapobj_str = self.wrapobj_str
        wrapobj = self.wrapobj
        wrapobj.update(dico)
        # Test if the dictionary has changed relative to self.wrapdic
        if self.wrapdic != wrapobj:
            self.wrapdic.update(dico)
            # Also update the wrapobj at the top level of the deepwrapdic
            self.other.updaters_queued.vardic.all_update()
            if wrapobj_str == 'self.vardic':
                if 'self.vardic' not in self.queue: self.queue.append('self.vardic')


class listwrap_queued:
    def __init__(self,other,wrapobj_str,initlev):
        self.other = other
        self.queue = other.updaters_queued.queue
        self.wrapobj_str = wrapobj_str
        wrapobj = eval('other.'+wrapobj_str.split('.')[1])
        self.wrapobj = wrapobj
        self.initlev = initlev
        self.wrapli = deepcopy(wrapobj)

    def __setslice__(self,ind1,ind2,into):
        other = self.other
        wrapobj_str = self.wrapobj_str
        wrapobj = self.wrapobj
        initlev = self.initlev
        lengo = len(self.wrapli)
        if ind2 >= lengo:
            print "ERROR: Assignment out of bounds of original list"
            return

        if self.wrapli[ind1:ind2] != into and wrapobj_str == 'self.foceqs':
            self.wrapli[ind1:ind2] = into
            if 'self.foceqs' not in self.queue: self.queue.append('self.foceqs')
        elif self.wrapli[ind1:ind2] != into and wrapobj_str == 'self.manss_sys':
            self.wrapli[ind1:ind2] = into
            if 'self.manss_sys' not in self.queue: self.queue.append('self.manss_sys')
        elif self.wrapli[ind1:ind2] != into and wrapobj_str == 'self.ssys_list':
            self.wrapli[ind1:ind2] = into
            if 'self.ssys_list' not in self.queue: self.queue.append('self.ssys_list')
    
    def __setitem__(self,ind,into):
        other = self.other
        wrapobj_str = self.wrapobj_str
        wrapobj = self.wrapobj
        initlev = self.initlev
        lengo = len(self.wrapli)
        if ind >= lengo:
            print "ERROR: Assignment out of bounds of original list"
            return

        if self.wrapli[ind] != into and wrapobj_str == 'self.foceqs':
            self.wrapli[ind] = into
            if 'self.foceqs' not in self.queue: self.queue.append('self.foceqs')
        elif self.wrapli[ind] != into and wrapobj_str == 'self.manss_sys':
            self.wrapli[ind] = into
            if 'self.manss_sys' not in self.queue: self.queue.append('self.manss_sys')
        elif self.wrapli[ind] != into and wrapobj_str == 'self.ssys_list':
            self.wrapli[ind] = into
            if 'self.ssys_list' not in self.queue: self.queue.append('self.ssys_list')

    def __getitem__(self,ind):
        lengo = len(self.wrapli)
        if ind >= lengo:
            print "ERROR: Assignment out of bounds of original list"
            return
        else:
            return self.wrapli[ind]

    def __repr__(self):
        return self.wrapli.__repr__()
    def __str__(self):
        return self.wrapli.__str__()


class matwrap_queued:
    def __init__(self,other,wrapobj_str,initlev):
        self.other = other
        self.queue = other.updaters_queued.queue
        self.wrapobj_str = wrapobj_str
        self.wrapobj = eval('other.'+wrapobj_str.split('.')[1])
        self.initlev = initlev
        if wrapobj_str == 'self.sigma':
            self.wrapmat = deepcopy(other.sigma)
            
    def __getattr__(self,attrname):
        return getattr(self.wrapmat,attrname)
    
    def __setitem__(self,ind,into):
        # ind is a tuple and into is (should be) a float
        other = self.other
        wrapob_str = self.wrapobj_str
        initlev = self.initlev
        if type(into) != type(1.111): into = float(into)
        if self.wrapmat[ind[0],ind[1]] != into and wrapob_str == 'self.sigma':
            self.wrapmat[ind[0],ind[1]] = into
            other.sigma[ind[0],ind[1]] = into
            if 'self.sigma' not in self.queue: self.queue.append('self.sigma')

   
class Process_Queue(object):
    def __init__(self,other=None):
        self.other = other
        self.queue = other.updaters_queued.queue
        self.initlev = other._initlev
        # The vardic
        self.vardic = deepcopy(other.updaters_queued.vardic.wrapobj)
        # The nlsubsdic
        if 'nlsubsdic' in dir(other):
            self.nlsubsdic = deepcopy(other.updaters_queued.nlsubsdic.wrapobj)
        # The paramdic
        self.paramdic = deepcopy(other.updaters_queued.paramdic.wrapobj)
        # The foceqs
        self.foceqs = deepcopy(other.updaters_queued.foceqs.wrapobj)
        # The manss_sys
        if 'manss_sys' in dir(other.updaters_queued):
            self.manss_sys = deepcopy(other.updaters_queued.manss_sys.wrapobj)
        # The ssys_list
        if 'ssys_list' in dir(other.updaters_queued):
            self.ssys_list = deepcopy(other.updaters_queued.ssys_list.wrapobj)
        # The sigma
        self.sigma = deepcopy(other.updaters_queued.sigma.wrapobj)
        
    def reinit(self):
        other = self.other
        # The vardic
        self.vardic = deepcopy(other.updaters_queued.vardic.wrapobj)
        # The nlsubsdic
        if 'nlsubsdic' in dir(other):
            self.nlsubsdic = deepcopy(other.updaters_queued.nlsubsdic.wrapobj)
        # The paramdic
        self.paramdic = deepcopy(other.updaters_queued.paramdic.wrapobj)
        # The foceqs
        self.foceqs = deepcopy(other.updaters_queued.foceqs.wrapobj)
        # The manss_sys
        if 'manss_sys' in dir(other.updaters_queued):
            self.manss_sys = deepcopy(other.updaters_queued.manss_sys.wrapobj)
        # The ssys_list
        if 'ssys_list' in dir(other.updaters_queued):
            self.ssys_list = deepcopy(other.updaters_queued.ssys_list.wrapobj)
        # The sigma
        self.sigma = deepcopy(other.updaters_queued.sigma.wrapobj)
        
        
    def __call__(self):
        self.reinit()
        queue = self.queue
        other = self.other
        initlev = self.initlev
        mesg = other._mesg
        
        if mesg:
            print "You have UPDATED the following items which are now PROCESSED:"
            for elem in queue:
                print elem
            print '================================================================'
        ##### THE INITS #####################
        other.init1(no_wrap=True) 
        if 'self.vardic' in queue:
            other.vardic = deepcopy(self.vardic)

        other.init1a()
        if 'self.nlsubsdic' in queue:
            for i1,elem in enumerate(other.nlsubs_raw1):
                other.nlsubs_raw1[i1][1] = deepcopy(self.nlsubsdic[other.nlsubs_raw1[i1][0]])
            for keyo in self.nlsubsdic.keys():
                other.nlsubsdic[keyo] = deepcopy(self.nlsubsdic[keyo])

        other.init1b(no_wrap=True)
        if 'self.paramdic' in queue:
            for keyo in self.paramdic.keys():
                other.paramdic[keyo] = deepcopy(self.paramdic[keyo])
        
        other.init1c(no_wrap=True)
        if 'self.foceqs' in queue:
            other.foceqs = deepcopy(self.foceqs)
        if 'self.manss_sys' in queue:
            other.manss_sys = deepcopy(self.manss_sys)
        if 'self.ssys_list' in queue:
            other.ssys_list = deepcopy(self.ssys_list)

        # Prepare DSGE model instance for manual SS computation
        other.init2()
        if initlev == 0:
            other.init_out()

        # Solve for SS automatically
        other.init3()
        if initlev == 1:
            other.init_out() 

        other.init4(no_wrap=True)
        if 'self.sigma' in queue:
            other.sigma = deepcopy(self.sigma)

        # Solve model dynamically            
        other.init5()
        if initlev == 2:
            other.init_out() 
        return



#
# Author: Pearu Peterson
# Created: February 2008
#
""" Provides Matrix support.
"""
from __future__ import division

__docformat__ = "restructuredtext"
__all__ = ['Matrix', 'MatrixBase', 'MatrixDict']

import random

from ..core import classes
from ..utils import SYMBOL, NUMBER, ADD, MUL, POW, MATRIX, MATRIX_DICT
from ..basealgebra import Algebra
from ..basealgebra.verbatim import Verbatim
from ..arithmetic.numbers import div, mpq

def is_sequence(obj):
    t = type(obj)
    if t is list or t is tuple:
        return True
    elif t is mpq:
        return False
    try:
        len(obj)
        return True
    except TypeError:
        return False

def is_integer(obj):
    t = type(obj)
    if t is int or t is long:
        return True
    try:
        int(obj)
        return True
    except TypeError:
        pass
    return False

def Matrix(*args, **kws):
    """ Construct a matrix instance.

    The following signatures are supported:

      Matrix(n)           - n x n matrix of zeros
      Matrix(n, random=True) - n x n matrix of random values belonging to [-10,10]
      Matrix(n, random=[l, u]) - n x n matrix of random values belonging to [l,u]
      Matrix(m, n)        - m x n matrix of zeros
      Matrix(n, m, random=True) - m x n matrix of random values belonging to [-10,10]
      Matrix(n, m, random=[l, u]) - m x n matrix of random values belonging to [l,u]
      Matrix(seq)         - n x 1 column matrix where n = len(seq)
      Matrix(seq, diagonal=True)    - n x n diagonal matrix where n = len(seq)
      Matrix(seq, permutation=True) - n x n permutation matrix where n = len(seq)
      Matrix(array)       - m x n matrix where m=len(array), n=max(len(array[:]))
      Matrix(m, n, dict)  - m x n matrix with nonzero elements defined by dict,
                            dict keys must be 2-tuples of integers.
      Matrix(m, n, seq)   - m x n matrix with elements in seq row-wise.
    """
    if len(args)==1:
        a = args[0]
        if isinstance(a, MatrixBase):
            return a
        elif isinstance(a, int):
            m, n = a, a
            data = {}
            interval = kws.get('random')
            if interval is not None:
                if interval==True:
                    interval = (-10, 10)
                for i in range(m):
                    for j in range(n):
                        num = random.randint(*interval)
                        if num:
                            data[i,j] = num                
        elif is_sequence(a):
            m = len(a)
            data = {}
            if True in map(is_sequence, a):
                n = 0
                for i,row in enumerate(a):
                    if is_sequence(row):
                        l = len(row)
                        n = max(n, l)
                        for j, x in enumerate(row):
                            if x:
                                data[i,j] = x
                    else:
                        n = max(n, 1)
                        if row:
                            data[i,0] = row
            else:
                n = m
                if kws.get('diagonal'):
                    for i, x in enumerate(a):
                        if x:
                            data[i, i] = x
                elif kws.get('permutation'):
                    for i,x in enumerate(a):
                        assert 0<=x<m,`x,m`
                        data[i, x] = 1
                else: # single column matrix
                    n = 1
                    for i,x in enumerate(a):
                        if x:
                            data[i, 0] = x
        else:
            raise TypeError('Matrix call with 1 argument: argument must be integer or a sequence, got %r' % (type(a)))
    elif len(args)==2:
        m, n = args
        data = {}
        if not (isinstance(m, int) and isinstance(n, int)):
            raise TypeError('Matrix call with 2 arguments: arguments must be integers, got %r, %r' % (type(m), type(n)))
        interval = kws.get('random')
        if interval is not None:
            if interval==True:
                interval = (-10, 10)
            for i in range(m):
                for j in range(n):
                    num = random.randint(*interval)
                    if num:
                        data[i,j] = num                
    elif len(args)==3:
        m, n, data = args
        flag = True
        if isinstance(data, dict):
            pass
        elif is_sequence(data):
            k = 0
            d = {}
            if data and is_sequence(data[k]):
                for i in range(m):
                    for j in xrange(n):
                        x = data[i][j]
                        if x:
                            d[i,j] = x
            else:
                for i in range(m):
                    for j in xrange(n):
                        x = data[k]
                        if x:
                            d[i,j] = x
                        k += 1
            data = d
        else:
            flag = False
        if not (isinstance(m, int) and isinstance(n, int) and flag):
            raise TypeError('Matrix call with 3 arguments: arguments must be integers and a dictionary|sequence, got %r, %r, %r'\
                            % (type(m), type(n), type(data)))
        
        
    else:
        raise TypeError('Matrix takes 1, 2, or 3 arguments, got %r' % (len(args)))

    return MatrixDict(MATRIX(m, n, MATRIX_DICT), data)


class MatrixBase(Algebra):
    """ Base class to matrix classes.

    Currently the following matrix classes are implemented:

      MatrixDict - matrix content is saved in a dictonary
    """

    @classmethod
    def convert(cls, data, typeerror=True):
        if isinstance(data, list):
            return Matrix(data)
        return super(CommutativeRing, cls).convert(data, typeerror=typeerror)

    @classmethod
    def random(cls, n, m, interval=(-10,10)):
        return Matrix(n, m, random=interval)

    def _str_column_fill(self, columns, j, rowiter1, sep = None, rowiter2 = None):
        col = []
        if rowiter1 is not None:
            for i in rowiter1:
                s = str(self[i,j])
                col.append(s)
        if sep is not None:
            col.append (sep)
        if rowiter2 is not None:
            for i in rowiter2:
                s = str(self[i,j])
                col.append(s)
        width = max(map(len,col)+[0])
        fmt = ' %'+str(width)+'s '
        col = [fmt % (s) for s in col]
        columns.append(col)

    def __str__(self, max_nrows=20, max_ncols=20):
        """ Return pretty string representation of a matrix.
        
        Parameters
        ----------
        max_nrows, max_ncols : int
          Specify the maximum number of rows and columns, respectively, that
          are included in the result.

        """
        rows, cols = self.rows, self.cols
        if self.head.is_diagonal:
            # XXX: what should be the output of diagonal view?
            self = self.M
        columns = []
        if cols>max_ncols and rows>max_nrows:
            for j in range((max_ncols+1)//2):
                self._str_column_fill (columns, j, xrange((max_nrows+1)//2), ':', xrange(rows-max_nrows//2, rows))
            columns.append([' .. ']*rows)
            for j in range(cols-(max_ncols)//2,cols):
                self._str_column_fill (columns, j, xrange((max_nrows+1)//2), ':', xrange(rows-max_nrows//2, rows))
        elif cols>max_ncols:
            for j in range((max_ncols+1)//2):
                self._str_column_fill(columns, j, xrange(rows))
            columns.append([' .. ']*rows)
            for j in range(cols-(max_ncols)//2,cols):
                self._str_column_fill (columns, j, xrange(rows))
        elif rows>max_nrows:
            for j in xrange(cols):
                self._str_column_fill(columns, j, xrange((max_nrows+1)//2), ':', xrange(rows-max_nrows//2, rows))
        else:
            for j in xrange(cols):
                self._str_column_fill (columns, j, xrange(rows), None, None)
        return '\n'.join([''.join(row).rstrip() for row in zip(*columns)])

    def __repr__ (self):
        rows, cols = self.rows, self.cols
        return 'Matrix (%s, %s, %r)' % (rows, cols, self.tolist())

    def __iadd__(self, other):
        raise NotImplementedError('%s must implement __iadd__' % (type(self)))

    def __imul__(self, other):
        raise NotImplementedError('%s must implement __imul__' % (type(self)))

    def __pos__(self):
        return self

    def __add__(self, other):
        ret = self.copy()
        ret += other
        return ret

    def __radd__(self, other):
        t = type(other)
        if t is list or t is tuple:
            return Matrix(other) + self
        # other is scalar
        ret = self.copy()
        ret += other
        return ret

    def __isub__(self, other):
        self += -other
        return self

    def __sub__(self, other):
        ret = self.copy()
        ret -= other
        return ret

    def __rsub__(self, other):
        return other + (-self)

    def __mul__(self, other):
        ret = self.copy()
        ret *= other
        return ret

    def __rmul__(self, other):
        t = type(other)
        if t is list or t is tuple:
            return Matrix(other) * self
        # other is scalar
        ret = self.copy()
        ret *= other
        return ret

    def __idiv__(self, other):
        self *= div(1, other)
        return self

    def __div__(self, other):
        return self * div(1, other)

    def __rdiv__(self, other):
        head, data = self.pair
        if head.is_array:
            return type(self)(head, dict([(ij,div(other,x)) for ij,x in data.iteritems()]))
        return other * self.inv()

    def __pow__(self, other):
        head, data = self.pair
        m, n = head.shape
        t = type(other)
        if t is int or t is long:
            if head.is_array:
                if other<0:
                    return type(self)(head, dict([(ij,div(1,x)**(-other)) for ij,x in data.iteritems()]))
                return type(self)(head, dict([(ij,x**other) for ij,x in data.iteritems()]))
            if head.is_diagonal:
                raise NotImplementedError(`head, str(head)`)
            if other < 0:
                return self.inv() ** (-other)
            if other==1:
                return self
            if other==0:
                return Matrix([1]*min(m,n), diagonal=True)
            if other==2:
                return self * self
            r = 1
            x = self
            while 1:
                if other & 1:
                    r *= x
                    other -= 1
                    if not other:
                        break
                x = x*x
                other //= 2
            return r
        return NotImplemented

class MatrixDict(MatrixBase):
    """ Implementation of matrix where elements are stored in a dictionary.
    """

    rows = property(lambda self: self.head.rows)
    cols = property(lambda self: self.head.cols)
    shape = property(lambda self: self.head.shape)
    is_square = property(lambda self: self.head.rows==self.head.cols)

    @property
    def is_lower(self):
        head, data = self.pair
        if head.is_transpose:
            return all(i<=j for i, j in data)
        return all(i>=j for i, j in data)

    @property
    def is_upper(self):
        head, data = self.pair
        if head.is_transpose:
            return all(i>=j for i, j in data)
        return all(i<=j for i, j in data)

    @property
    def is_orthogonal(self):
        if not self.is_square:
            return False
        return (self*self.T).is_identity

    @property
    def is_identity(self):
        head, data = self.pair
        m, n = head.shape
        if m!=n or len(data)!=n:
            return False
        return all(i==j and x==1 for (i,j),x in data.iteritems())

    @property
    def is_empty (self):
        head, data = self.pair
        m, n = head.shape
        return m == n == 0

    @property
    def is_zero(self):
        head, data = self.pair
        if not data:
            return True
        m, n = head.shape
        for (i,j),x in data.iteritems ():
            if (i<0 or i>=m): continue
            if (j<0 or j>=n): continue
            return False
        return True

    @property
    def is_row_echelon_form(self):
        """ Return True if matrix is in row echelon form, that is,

        (i) All nonzero rows (rows with at least one nonzero element)
        are above any rows of all zeroes, and
        
        (ii) The leading coefficient (the first nonzero number from
        the left, also called the pivot) of a nonzero row is always
        strictly to the right of the leading coefficient of the row
        above it.
        """
        head, data = self.pair
        m, n = head.shape
        expect_i, pivot_i, pivot_j = 0, -1, -1
        for (i,j) in sorted(data.keys()):
            if pivot_i==i:
                continue
            if expect_i!=i or pivot_j>=j:
                return False
            pivot_i, pivot_j, expect_i = i,j,i+1
        return True

    @property
    def is_row_canonical_form(self):
        """ Return True if matrix is in reduced row echelon form, that
        is,

        (i) All nonzero rows (rows with at least one nonzero element)
        are above any rows of all zeroes, and
        
        (ii) The leading coefficient (the first nonzero number from
        the left, also called the pivot) of a nonzero row is always
        strictly to the right of the leading coefficient of the row
        above it.

        (iii) Every leading coefficient is 1 and is the only nonzero
        entry in its column.
        """
        head, data = self.pair
        m, n = head.shape
        expect_i, pivot_i, pivot_j = 0,-1, -1
        for (i,j) in sorted(data.keys()):
            if pivot_i==i:
                continue
            if expect_i!=i or pivot_j>=j:
                return False
            x = data[i,j]
            if x!=1:
                return False
            if not self[:i, j].is_zero:
                return False
            pivot_i, pivot_j, expect_i = i,j,i+1
        return True

    @property
    def T(self):
        """ Return transposed view of a matrix.
        """
        head, data = self.pair
        return type(self)(head.T, data)

    @property
    def A(self):
        """ Return array view of a matrix.
        """
        head, data = self.pair
        if head.is_array:
            return self
        newhead = head.A
        if newhead is head:
            return self
        return type(self)(newhead, data)

    @property
    def M(self):
        """ Return matrix view of a matrix.
        """
        head, data = self.pair
        if not (head.is_array or head.is_diagonal):
            return self
        newhead = head.M
        if newhead is head:
            return self
        return type(self)(newhead, data)

    @property
    def D(self):
        """ Return diagonal view of a matrix.
        """
        head, data = self.pair
        if head.is_diagonal:
            return self
        newhead = head.D
        if newhead is head:
            return self
        return type(self)(newhead, data)

    @property
    def I(self):
        """ Return inverse matrix.
        """
        return self.inv()

    def resize(self, m, n):
        """ Return resized view of a matrix.

        Notes:

         - Use crop() to remove elements out-of-matrix bounds.
         - If matrix is not writable then matrix data is copied.
        """
        head, data = self.pair
        p, q = head.shape
        if p==m and q==n:
            return self
        newhead = MATRIX(m, n, head.storage)
        if not self.is_writable:
            data = dict(data)
        return type(self)(newhead, data)

    def tolist(self):
        """Convert matrix to a list of lists."""
        rows, cols = self.head.shape
        return [[self[i,j] for j in xrange(cols)] for i in xrange(rows)]
    
    def flatten(self):
        rows, cols = self.head.shape
        for i in xrange (rows):
            for j in xrange (cols):
                yield self[i,j]

    def _get_diagonal(self, key):
        head, data = self.pair
        rows, cols = head.shape
        t = type(key)
        if t is int or t is long:
            if key<0:
                m = min(rows, cols) + key
            else:
                m = min(rows, cols) - key
            d = {}
            if head.is_transpose:
                key = -key
            for (i,j),x in data.items():
                if j-i==key:
                    k = min(i,j)
                    d[k,0] = x
            return Matrix(m, 1, d)
        elif is_integer(key):
            return self._get_diagonal[int(key)]
        raise NotImplementedError(`head, key, t`)

    def __getitem__(self, key):
        tkey = type(key)
        head, data = self.pair
        if head.is_diagonal:
            return self._get_diagonal(key)
        if tkey is tuple:
            i, j = key
            ti, tj = type(i), type(j)
            if ti is int and i<0:
                i += head.rows
            if tj is int and j<0:
                j += head.cols
            if ti is int and tj is int:
                if head.is_transpose:
                    key = j, i
                elif head.is_diagonal:
                    raise NotImplementedError(`head, str(head)`)
                else:
                    key = i, j
                if i>=head.rows or j>=head.cols:
                    raise IndexError (`i,j,head.cols,head.rows`)
                return data.get(key, 0)
            if ti is slice:
                row_indices = dict([(i0,k) for k,i0 in enumerate(xrange(*i.indices(head.rows)))])
            elif ti is int:
                row_indices = {i:0}
            elif ti is tuple:
                row_indices = dict([(i0,k) for k,i0 in enumerate(i)])                
            elif is_integer(i):
                return self[int(i), j]
            else:
                raise IndexError('first index must contain int or slice or tuple, got %s' % ((`ti`)))
            if tj is slice:
                col_indices = dict([(j0,k) for k,j0 in enumerate(xrange(*j.indices(head.cols)))])
            elif tj is int:
                col_indices = {j:0}
            elif tj is tuple:
                col_indices = dict([(j0,k) for k,j0 in enumerate(j)])
            elif is_integer(j):
                return self[i, int (j)]
            else:
                raise IndexError('second index must contain int or slice or tuple, got %s' % ((`tj`)))
            newdata = {}
            if head.is_transpose:
                for (j,i), x in data.items():
                    ki = row_indices.get(i)
                    if ki is not None:
                        kj = col_indices.get(j)
                        if kj is not None:
                            newdata[ki, kj] = x
            elif head.is_diagonal:
                raise NotImplementedError(`head, str(head)`)
            else:
                for (i,j), x in data.items():
                    ki = row_indices.get(i)
                    if ki is not None:
                        kj = col_indices.get(j)
                        if kj is not None:
                            newdata[ki, kj] = x
            return Matrix(len(row_indices), len(col_indices), newdata)
        elif tkey is int:
            if key>=head.rows:
                raise IndexError (`key, head.rows`)
            return self[key, :]
        elif tkey is slice:
            return self[key, :]
        elif tkey is tuple:
            return self[key, :]
        elif is_integer(key):
            return self[int(key)]

        raise IndexError('index must be int, slice, or tuple, got %s' % (tkey))

    def _set_diagonal(self, key, value):
        head, data = self.pair
        assert type(key) in [int, long],`type(key)`
        rows, cols = head.shape
        if head.is_transpose:
            key = -key
        if key<0:
            k = min(rows, cols) + key
        else:
            k = min(rows, cols) - key
        if isinstance(value, MatrixBase):
            m, n = value.head.shape
            assert n==1,`m,n`
            assert m==k,`m,k,rows,cols`
            if key<0:
                for (i,j),x in value.data.items():
                    data[i-key,i] = x
            else:
                for (i,j),x in value.data.items():
                    data[i,i+key] = x
        else:
            if key < 0:
                for i in range(k):
                    data[i-key, i] = value
            else:
                for i in range(k):
                    data[i, i+key] = value

    def __setitem__(self, key, value):
        if not self.is_writable:
            raise TypeError('Matrix content is read-only')
        if isinstance(value, list):
            self[key] = Matrix(value)
            return

        head, data = self.pair
        if head.is_diagonal:
            return self._set_diagonal(key, value)
        tkey = type(key)
        if tkey is int or tkey is slice:
            if head.is_diagonal:
                raise NotImplementedError(`head, str(head)`)
            self[key, :] = value
            return
        i, j = key
        ti, tj = type(i), type(j)
        if ti is int and i<0:
            i += head.rows
        if tj is int and j<0:
            j += head.cols
        if ti is int and tj is int:
            if head.is_transpose:
                key = j, i
            elif head.is_diagonal:
                raise NotImplementedError(`head, str(head)`)
            else:
                key = i, j
            if value:
                data[key] = value
            else:
                try:
                    del data[key]
                except KeyError:
                    pass
            return
        if ti is slice and tj is int:
            row_indices = [(i0,k) for k,i0 in enumerate(xrange(*i.indices(head.rows)))]
            col_indices = [(j,0)]
        elif ti is int and tj is slice:
            row_indices = [(i,0)]
            col_indices = [(j0,k) for k,j0 in enumerate(xrange(*j.indices(head.cols)))]
        elif ti is slice and tj is slice:
            row_indices = [(i0,k) for k,i0 in enumerate(xrange(*i.indices(head.rows)))]
            col_indices = [(j0,k) for k,j0 in enumerate(xrange(*j.indices(head.cols)))]
        elif is_integer(i) and tj in (int, slice):
            
            return self.__setitem__((int(i),j), value)
        elif is_integer(j) and ti in (int, slice):
            return self.__setitem__((i,int(j)), value)
        else:
            raise IndexError('tuple index must contain int or slice but got %r' % ((ti, tj),))
        if isinstance(value, MatrixBase):
            m, n = value.head.shape
            assert len(row_indices)==m,`len(row_indices),m`
            assert len(col_indices)==n,`len(col_indices),n`
            if head.is_transpose:
                for i,ki in row_indices:
                    for j,kj in col_indices:
                        v = value[ki, kj]
                        if v:
                            data[j, i] = v
                        else:
                            try:
                                del data[j, i]
                            except KeyError:
                                pass
            elif head.is_diagonal:
                raise NotImplementedError(`head, str (head)`)
            else:
                for i,ki in row_indices:
                    for j,kj in col_indices:
                        v = value[ki, kj]
                        if v:
                            data[i, j] = v
                        else:
                            try:
                                del data[i, j]
                            except KeyError:
                                pass
        else:
            if value:
                if head.is_transpose:
                    for i,ki in row_indices:
                        for j,kj in col_indices:
                            data[j, i] = value
                elif head.is_diagonal:
                    raise NotImplementedError(`head`)
                else:
                    for i,ki in row_indices:
                        for j,kj in col_indices:
                            data[i, j] = value
            else:
                if head.is_transpose:
                    for i,ki in row_indices:
                        for j,kj in col_indices:
                            try:
                                del data[j, i]
                            except KeyError:
                                pass
                elif head.is_diagonal:
                    raise NotImplementedError(`head`)
                else:
                    for i,ki in row_indices:
                        for j,kj in col_indices:
                            try:
                                del data[i, j]
                            except KeyError:
                                pass
    
    def copy(self):
        """ Return a copy of a matrix.
        """
        head, data = self.pair
        return MatrixDict(head, dict(data))


    def __array__(self):
        """ Return matrix as numpy array.
        """
        import numpy
        head, data = self.pair
        storage = head.storage
        shape = head.shape
        r = numpy.zeros(shape,dtype=object)
        if head.is_transpose:
            for (j, i),e in data.items():
                r[i][j] = e
        elif head.is_diagonal:
            raise NotImplementedError(`head`)
        else:
            for (i,j),e in data.items():
                r[i][j] = e
        return r

    def __neg__(self):
        head, data = self.pair
        return MatrixDict(head, dict([(index,-element) for index, element in data.items()]))

    def inv_l(self):
        """ Return inverse of lower triangular matrix L.
        """

        M,N = self.head.shape
        linv = Matrix(M,N)
        assert M==N,`M,N`
        for j in xrange(N):
            linv[j,j] = div(1, self[j, j])
            for i in xrange(j+1,N):
                r = 0
                for k in range(j,i):
                    r = r - self[i, k] * linv[k, j]
                linv[i, j] = div(r, self[i, i])
        return linv

    def inv(self):
        """ Return inverse of a square matrix.
        """
        head, data = self.pair
        m, n = head.shape
        assert m==n,`m,n`
        if head.is_transpose:
            d = {}
            for (i,j),x in data.items():
                d[j,i] = x
        elif head.is_diagonal:
            raise NotImplementedError(`head`)
        else:
            d = dict(data)
        for i in range(m):
            d[i,i+m] = 1
        a = MatrixDict(MATRIX(m, 2*m, MATRIX_DICT), d)
        # XXX: catch singular matrices
        b = a.gauss_jordan_elimination(overwrite=True)
        return b[:,m:]

    def solve(self, rhs):
        """ Solve a system of linear equations A * x = rhs

        where A is square matrix. Alternative usage::

          A // rhs -> x
        
        For example::

          Matrix([[1,2], [3,4]]) // [1,2] -> Matrix([[0],[1/2]])
        
        See also
        --------
        solve_null

        """
        t = type(rhs)
        if t is tuple or t is list:
            rhs = Matrix(rhs)
        head, data = self.pair
        m, n = head.shape
        assert m==n,`m,n`
        if head.is_transpose:
            d = {}
            for (i,j), x in data.items():
                d[j,i] = x
        elif head.is_diagonal:
            raise NotImplementedError(`head`)
        else:
            d = dict(data)
        rhead, rdata = rhs.pair
        p, q = rhead.shape
        assert p==m,`p,m`
        if rhead.is_transpose:
            for (j,i),x in rdata.items():
                d[i,j+n] = x
        else:
            for (i,j),x in rdata.items():
                d[i,j+n] = x
        a = MatrixDict(MATRIX(m, n+q, MATRIX_DICT), d)
        b = a.gauss_jordan_elimination(overwrite=True)
        return b[:,m:]

    __floordiv__ = solve

    def normal(self):
        d = {}
        head, data = self.pair
        for ij, x in data.iteritems():
            if hasattr(x, 'normal'):
                d[ij] = x.normal()
            else:
                d[ij] = x
        return type(self)(head, d)

    def expand(self):
        d = {}
        head, data = self.pair
        for ij, x in data.iteritems():
            if hasattr(x, 'expand'):
                d[ij] = x.expand()
            else:
                d[ij] = x
        return type(self)(head, d)

    def subs (self, subexpr, newexpr = None):
        data = {}
        for key, value in self.data.items():
            if isinstance (value, classes.Expr):
                value = value.subs(subexpr, newexpr)
            if value:
                data[key] = value
        return type(self)(self.head, data)

    @property
    def rank(self):
        """ Return row-rank of a matrix.
        """
        return self.gauss_jordan_elimination().rows

    @property
    def nullity (self):
        return self.cols - self.rank

    def solve_null(self, labels = None, check=False):
        """ Solve a system of homogeneous equations: A * x = 0

        where A is m x n matrix, x is n vector.

        Parameters
        ----------
        labels : {list, tuple}
          A list of column symbols.
        check : bool
          When True the check the definitions and raise a runtime
          exception on invalid results.

        Returns
        -------
        xd : dict
          A dictionary of null solutions. xd keys are labels of
          columns and xd values are rows of A null space basis
          matrix. That is, let us define::

            ker = Matrix([xd[l].tolist()[0] for l in labels])

          then::

            A * ker = 0

          or equivalently, if

            x = ker * Matrix(labels)

          then

            A * x = 0.

        dependent, independent : list
          Lists of dependent and independent column labels, respectively.

          If
            x_dep = [sum(dot(xd[l], independent)) for l in labels]
          then
            A * x_dep = 0

        See also
        --------
        solve
        """
        m, n = self.head.shape
        if labels is None:
            labels = range(n)
        gj, (dep, indep) = self.gauss_jordan_elimination(labels = labels)
        xd = {}
        rank = gj.rows
        nullity = n - rank
        for i, label in enumerate (dep):
            xd[label] = -gj[i, rank:]
        for i, label in enumerate (indep):
            xd[label] = Matrix(1,nullity, {(0,i):1})
        if check:
            ker = Matrix([xd[l].tolist () [0] for l in labels])
            null = self * ker
            if not null.is_zero:
                raise RuntimeError('%s.solve_null: A*ker=0 test failed' % (self.__class__.__name__))
            c = Matrix([classes.Calculus('X%s'%(label)) for label in indep])
            x = ker * c
            null = self*x
            if not null.is_zero:
                raise RuntimeError('%s.solve_null: A*(ker*indep)=0 test failed' % (self.__class__.__name__))

        return xd, dep, indep

    def gram_schmidt_process(self):
        """ Orthogonalized matrix columns using Gram-Schmidt process.

        Returns
        -------
        U : Matrix

        Notes
        -----
        U columns are orthogonal (but not orthonormal). U * self
        produces upper triangular matrix.
        """
        m, n = self.head.shape
        U = Matrix(m,n)
        k1 = 0
        l = []
        for k in range(n):
            ak = self[:,k]
            uk = ak
            flag = True
            for j in range(k1):
                uj = U[:,j]
                cdenom = (uj.T*uj)[0,0]
                c = div((uj.T*ak)[0,0], cdenom)
                uk = uk - c * uj
            if not uk.is_zero:
                U[:,k1] = uk
                k1 += 1
                l.append(k)
        U = U.resize(m, k1)
        return U, l

def switch_dep_indep_variables (xd, dep, indep, dep_label, indep_label):
    m = rank = len (dep)
    nullity = len (indep)
    n = m + nullity
    u = Matrix(m,n)
    for i in range(m):
        u[i,i] = 1
    for i,label in enumerate(dep):
        u[i,m:] = -xd[label]
    j1 = dep.index (dep_label)
    j2 = m + indep.index (indep_label)
    dep, indep = dep[:], indep[:]
    dep[j1] = indep_label
    indep[j2-m] = dep_label
    u.swap_cols (j1, j2)
    gj = u[:].gauss_jordan_elimination(overwrite=True)
    new_xd = {}
    for i in range (m):
        label = dep[i]
        new_xd[label] = -gj[i, rank:]
    for i in range (nullity):
        label = indep[i]
        new_xd[label] = Matrix(1, nullity, {(0,i):1})
    return new_xd, dep, indep

from .matrix_operations import MATRIX_DICT_iadd, MATRIX_DICT_imul
from .linalg import (MATRIX_DICT_swap_rows, MATRIX_DICT_swap_cols,
                     MATRIX_DICT_lu, MATRIX_DICT_crop,
                     MATRIX_DICT_gauss_jordan_elimination,
                     MATRIX_DICT_trace,
                     MATRIX_DICT_get_gauss_jordan_elimination_operations,
                     MATRIX_DICT_apply_row_operations)
from .linalg_determinant import MATRIX_DICT_determinant
from .linalg_lp import MATRIX_DICT_LP_solve

MatrixDict.__iadd__ = MATRIX_DICT_iadd
MatrixDict.__imul__ = MATRIX_DICT_imul
MatrixDict.swap_rows = MATRIX_DICT_swap_rows
MatrixDict.swap_cols = MATRIX_DICT_swap_cols
MatrixDict.crop = MATRIX_DICT_crop
MatrixDict.lu = MATRIX_DICT_lu
MatrixDict.gauss_jordan_elimination = MATRIX_DICT_gauss_jordan_elimination
MatrixDict.det = MATRIX_DICT_determinant
MatrixDict.trace = MATRIX_DICT_trace
MatrixDict.get_gauss_jordan_elimination_operations = MATRIX_DICT_get_gauss_jordan_elimination_operations
MatrixDict.apply_row_operations = MATRIX_DICT_apply_row_operations
MatrixDict.LP_solve = MATRIX_DICT_LP_solve



# Parallel Python Software: http://www.parallelpython.com
# Copyright (c) 2005-2012, Vitalii Vanovschi
# All rights reserved.
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#    * Redistributions of source code must retain the above copyright notice,
#      this list of conditions and the following disclaimer.
#    * Redistributions in binary form must reproduce the above copyright
#      notice, this list of conditions and the following disclaimer in the
#      documentation and/or other materials provided with the distribution.
#    * Neither the name of the author nor the names of its contributors
#      may be used to endorse or promote products derived from this software
#      without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
# THE POSSIBILITY OF SUCH DAMAGE.
"""
Parallel Python Software, PP Worker

http://www.parallelpython.com - updates, documentation, examples and support
forums
"""
import sys
import os
import StringIO
import cPickle as pickle
import pptransport

copyright = "Copyright (c) 2005-2012 Vitalii Vanovschi. All rights reserved"
version = "1.6.2"


def preprocess(msg):
    fname, fsources, imports = pickle.loads(msg)
    fobjs = [compile(fsource, '<string>', 'exec') for fsource in fsources]
    for module in imports:
        try:            
            if not module.startswith("from ") and not module.startswith("import "):
                module = "import " + module
            exec module
            globals().update(locals())
        except:
            print "An error has occured during the module import"
            sys.excepthook(*sys.exc_info())
    return fname, fobjs

class _WorkerProcess(object):

    def __init__(self):
        self.hashmap = {}
        self.e = sys.__stderr__
        self.sout = StringIO.StringIO()
#        self.sout = open("/tmp/pp.debug","a+")
        sys.stdout = self.sout
        sys.stderr = self.sout
        self.t = pptransport.CPipeTransport(sys.stdin, sys.__stdout__)
        self.t.send(str(os.getpid()))
        self.pickle_proto = int(self.t.receive())

    def run(self):
        try:
            #execution cycle
            while 1:
                __fname, __fobjs = self.t.creceive(preprocess)

                __sargs = self.t.receive()

                for __fobj in __fobjs:
                    try:
                        exec __fobj
                        globals().update(locals())
                    except:
                        print "An error has occured during the " + \
                              "function import"
                        sys.excepthook(*sys.exc_info())

                __args = pickle.loads(__sargs)
            
                __f = locals()[__fname]
                try:
                    __result = __f(*__args)
                except:
                    print "An error has occured during the function execution"
                    sys.excepthook(*sys.exc_info())
                    __result = None

                __sresult = pickle.dumps((__result, self.sout.getvalue()),
                        self.pickle_proto)

                self.t.send(__sresult)
                self.sout.truncate(0)
        except:
            print "A fatal error has occured during the function execution"
            sys.excepthook(*sys.exc_info())
            __result = None
            __sresult = pickle.dumps((__result, self.sout.getvalue()),
                    self.pickle_proto)
            self.t.send(__sresult)


if __name__ == "__main__":
        # add the directory with ppworker.py to the path
        sys.path.append(os.path.dirname(__file__))
        wp = _WorkerProcess()
        wp.run()

# Parallel Python Software: http://www.parallelpython.com



""" Provides CommutativeRing class.
"""

__docformat__ = "restructuredtext"
__all__ = ['CommutativeRing']

from .algebra import Algebra

class CommutativeRing(Algebra):
    """ Base class to commutative rings.

    Derived classes may redefine the following methods::
    
      Symbol(cls, obj)
      Number(cls, obj)
      Add(cls, *seq)
      Mul(cls, *seq),
      Pow(cls, base, exponent)
      Terms(cls, *seq)
      Factors(cls, *seq)
      as_Add_args(self)
      as_Mul_args(self)
      as_Pow_args(self),
      as_Terms_args(self)
      as_Factors_args(self)
    """
    __slots__ = ['_symbols']
    _symbols = None


    @classmethod
    def Pos(cls, arg): return +arg

    @classmethod
    def Neg(cls, arg): return -arg

    @classmethod
    def Add(cls, *seq):
        """ Compute sum over seq containing algebra elements.
        """
        raise NotImplementedError('%s must define classmethod Add'    #pragma NO COVER
                                  % (cls.__name__))                   #pragma NO COVER

    @classmethod
    def Sub(cls, *seq):
        """ Compute ``seq[0] - Add(*seq[0])``.
        """
        if seq:
            return seq[0] - cls.Add(*seq[1:])
        return cls.zero
    
    @classmethod
    def Mul(cls, *seq):
        """ Compute product over seq containing algebra elements.
        """
        raise NotImplementedError('%s must define classmethod Mul'    #pragma NO COVER
                                  % (cls.__name__))                   #pragma NO COVER

    @classmethod
    def Div(cls, *seq):
        """ Compute ``seq[0] / Mul(*seq[0])``.
        """
        if seq:
            return seq[0] / cls.Mul(*seq[1:])
        return cls.one

    @classmethod
    def Pow(cls, base, exponent):
        """ Compute power from base and exponent.
        
        Argument base must be an algebra element and exponent must be
        an element of exponent algebra.
        """
        raise NotImplementedError('%s must define classmethod Pow'     #pragma NO COVER
                                  % (cls.__name__))                    #pragma NO COVER

    @classmethod
    def Log(cls, arg, base=None):
        """ Compute logarithm of arg in base.

        Argument arg must be an element of exponent algebra and base
        is an element of an algebra.
        """
        raise NotImplementedError('%s must define classmethod Pow'     #pragma NO COVER
                                  % (cls.__name__))                    #pragma NO COVER

    @classmethod
    def Terms(cls, *seq):
        """ Compute sum over seq containing pairs ``(element, coefficient)``.

        ``element``-s must belong to algebra, ``coefficient``-s must
        belong to the coefficient algebra.
        """
        raise NotImplementedError('%s must define classmethod Terms'   #pragma NO COVER
                                  % (cls.__name__))                    #pragma NO COVER

    @classmethod
    def Factors(cls, *seq):
        """ Compute product over seq containing pairs ``(element, exponent)``.

        ``element``-s must belong to algebra, ``exponent``-s must belong
        to the exponent algebra.
        """
        raise NotImplementedError('%s must define classmethod Factors' #pragma NO COVER
                                  % (self.__class__.__name__))         #pragma NO COVER

    def as_Add_args(self):
        """ Return a sequence such that ``Add(*self.as_Add_args()) == self``
        """
        raise NotImplementedError('%s must define method as_Add_args'  #pragma NO COVER
                                  % (self.__class__.__name__))         #pragma NO COVER
    
    def as_Mul_args(self):
        """ Return a sequence such that ``Mul(*self.as_Mul_args()) == self``
        """
        raise NotImplementedError('%s must define method as_Mul_args'  #pragma NO COVER
                                  % (self.__class__.__name__))         #pragma NO COVER
    
    def as_Pow_args(self):
        """ Return a 2-tuple such that ``Pow(*self.as_Pow_args()) == self``
        """
        raise NotImplementedError('%s must define method as_Pow_args'  #pragma NO COVER
                                  % (self.__class__.__name__))         #pragma NO COVER

    def as_Log_args(self):
        """ Return a 2-tuple such that ``Log(*self.as_Log_args()) == self``
        """
        raise NotImplementedError('%s must define method as_Log_args'  #pragma NO COVER
                                  % (self.__class__.__name__))         #pragma NO COVER

    def as_Terms_args(self):
        """ Return a sequence such that ``Terms(*self.as_Terms_args()) == self``
        """
        raise NotImplementedError('%s must define method as_Terms_args' #pragma NO COVER
                                  % (self.__class__.__name__))          #pragma NO COVER

    def as_Factors_args(self):
        """ Return a sequence such that ``Factors(*self.as_Factors_args()) == self``
        """
        raise NotImplementedError('%s must define method as_Factors_args' #pragma NO COVER
                                  % (self.__class__.__name__))            #pragma NO COVER

    def __pos__(self):
        return self

    def __neg__(self):
        return self.Mul(self, self.convert(-1))

    def __add__(self, other):
        other = self.convert(other, False)
        if other is NotImplemented:
            return NotImplemented
        return self.Add(self, other)

    def __radd__(self, other):
        other = self.convert(other, False)
        if other is NotImplemented:
            return NotImplemented
        return self.Add(other, self)

    def __sub__(self, other):
        other = self.convert(other, False)
        if other is NotImplemented:
            return NotImplemented
        return self + (-other)

    def __rsub__(self, other):
        other = self.convert(other, False)
        if other is NotImplemented:
            return NotImplemented
        return other + (-self)

    def __mul__(self, other):
        other = self.convert(other, False)
        if other is NotImplemented:
            return NotImplemented
        return self.Mul(self, other)

    def __rmul__(self, other):
        other = self.convert(other, False)
        if other is NotImplemented:
            return NotImplemented
        return self.Mul(other, self)

    def __div__(self, other):
        other = self.convert(other, False)
        if other is NotImplemented:
            return NotImplemented
        if isinstance(other, (int, long)):
            other = self.Number(other)
        return self * other ** (-1)

    def __rdiv__(self, other):
        other = self.convert(other, False)
        if other is NotImplemented:
            return NotImplemented
        return other * self ** (-1)

    def __truediv__(self, other):
        other = self.convert(other, False)
        if other is NotImplemented:
            return NotImplemented
        return self * other ** (-1)

    def __rtruediv__(self, other):
        other = self.convert(other, False)
        if other is NotImplemented:
            return NotImplemented
        return other * self ** (-1)

    def __pow__(self, other):
        other = self.convert_exponent(other, False)
        if other is NotImplemented:
            return NotImplemented
        return self.Pow(self, other)

    def __rpow__(self, other):
        other = self.convert(other, False)
        if other is NotImplemented:
            return NotImplemented
        return self.Pow(other,  self.convert_exponent(self))

    def _matches(pattern, expr, repl_dict, wild_info):
        pfunc = pattern.func
        efunc = expr.func
        Add = pattern.Add
        Mul = pattern.Mul
        if pfunc==Add:
            wild_part = []
            exact_part = []
            for a in pattern.args:
                if a.symbols.intersection(wild_info[0]):
                    if a.head is SYMBOL:
                        wild_part.append(a)
                    else:
                        wild_part.insert(0, a)
                else:
                    exact_part.append(a)
            if exact_part:
                expr = expr - Add(*exact_part)
            if len(wild_part)==1:
                return wild_part[0].matches(expr, repl_dict, wild_info)
            expr_args = expr.as_Add_args() + [pattern.zero]
            for i in range(len(wild_part)):
                w = wild_part[i]
                w_rest = Add(*(wild_part[:i]+wild_part[i+1:]))
                for e in expr_args:
                    r = w.matches(e, repl_dict, wild_info)
                    if r is not None:
                        r1 = w_rest.subs(r.items()).matches(expr-e, r, wild_info)
                        if r1 is not None:
                            return r1
        elif pfunc==Mul:
            wild_part = []
            exact_part = []
            for a in pattern.args:
                if a.symbols.intersection(wild_info[0]):
                    if a.head is SYMBOL:
                        wild_part.append(a)
                    else:
                        wild_part.insert(0, a)
                else:
                    exact_part.append(a)
            if exact_part:
                expr = expr / Mul(*exact_part)
            if len(wild_part)==1:
                return wild_part[0].matches(expr, repl_dict, wild_info)
            expr_args = expr.as_Mul_args() + [pattern.one]
            for i in range(len(wild_part)):
                w = wild_part[i]
                w_rest = Mul(*(wild_part[:i]+wild_part[i+1:]))
                for e in expr_args:
                    r = w.matches(e, repl_dict, wild_info)
                    if r is not None:
                        r1 = w_rest.subs(r.items()).matches(expr/e, r, wild_info)
                        if r1 is not None:
                            return r1
        elif pfunc == efunc:
            return pattern._matches_seq(pattern.args, expr.args, repl_dict, wild_info)
        return

from ..utils import NUMBER, SYMBOL




from ..core import init_module
from .base import AtomicHead, Expr, heads_precedence

init_module.import_heads()
init_module.import_lowlevel_operations()

class DifferentialHead(AtomicHead):
    
    def base_exp(self, cls, expr):
        return expr, 1

    def term_coeff(self, cls, expr):
        return expr, 1

    def commutative_mul_number(self, cls, lhs, rhs):
        return term_coeff_new(cls, (lhs, rhs))

    def add(self, cls, lhs, rhs):
        h,d = rhs.pair
        if h is NUMBER:
            if d==0: return lhs
            return cls(TERM_COEFF_DICT, {lhs:1, cls(NUMBER,1):d})
        elif h is self:
            if lhs.data == d:
                return cls(TERM_COEFF, (lhs, 2))
            return cls(TERM_COEFF_DICT, {lhs:1, rhs:1})
        elif h is TERM_COEFF:
            t,c = d
            if lhs==t:
                return term_coeff_new(cls, (t, c+1))
            return cls(TERM_COEFF_DICT, {t:c, lhs:1})
        elif h is TERM_COEFF_DICT:
            data = d.copy()
            dict_add_item(cls, data, lhs, 1)
            return term_coeff_dict_new(cls, data)
        raise NotImplementedError(`self, h`)

    def commutative_mul(self, cls, lhs, rhs):
        h, d = rhs.pair
        if h is NUMBER:
            return cls.commutative_mul_number(cls, lhs, d)
        if h is self:
            if lhs.data==d:
                return cls(POW, (lhs, 2))
            return cls(BASE_EXP_DICT, {lhs:1, rhs:1})
        if h is POW:
            base, exp = d
            if base==lhs:
                return pow_new(cls, (base, exp + 1))
            return cls(BASE_EXP_DICT, {base:exp, lhs:1})
            
        raise NotImplementedError(`self, h`)

    def pow(self, cls, base, exp):
        return pow_new(cls, (base, exp))

    pow_number = pow

class DiffHead(DifferentialHead):

    """
    FunctionAlgebra(DIFF, x) - differential with respect to x
    x is symbol
    """
    
    def __repr__(self): return 'DIFF'

    def is_data_ok(self, cls, data):
        if isinstance(data, Expr):
            if data.head is SYMBOL: return
            return 'data must be with SYMBOL head but got %r' % (data.head)
        return 'data must be symbol but got %s' % (type(data))

    def data_to_str_and_precedence(self, cls, data):
        return SUBSCRIPT.data_to_str_and_precedence(cls, (Expr(SYMBOL, 'D'), (data,)))

    def diff_apply(self, cls, data, diff, expr):
        return expr.head.diff(type(expr), expr.data, expr, data.data, 1)

class FDiffHead(DifferentialHead):

    """
    FunctionAlgebra(FDIFF, x) - differential with respect to x-th argument
    x is number or symbol
    """
    
    def __repr__(self): return 'FDIFF'

    def is_data_ok(self, cls, data):
        if isinstance(data, Expr):
            if data.head is SYMBOL or data.head is NUMBER: return
            return 'data must be with SYMBOL|NUMBER head but got %r' % (data.head)
        return 'data must be symbol or number but got %s' % (type(data))

    def data_to_str_and_precedence(self, cls, data):
        return SUBSCRIPT.data_to_str_and_precedence(cls, (Expr(SYMBOL, 'FD'), (data,)))

    def diff_apply(self, cls, data, diff, expr):
        return expr.head.fdiff(type(expr), expr.data, expr, data.data, 1)


DIFF = DiffHead()
FDIFF = FDiffHead()



"""Provides base class for extended numbers: Infinity.
"""
__docformat__ = "restructuredtext"
__all__ = ['Infinity']

from ..core import init_module, Expr
init_module.import_heads()

from .numbers import realtypes, complextypes, mpqc, numbertypes, div

def is_undefined(obj):
    if isinstance(obj, Infinity):
        return not obj.data
    return False

def is_zoo(obj):
    if isinstance(obj, Infinity):
        return is_undefined(obj.data)
    return False

class Infinity(object):
    """ Base class for directional infinities.

    The formal definition of directional infinity is

    ::

      Infinity(direction) == oo * direction

    where ``oo`` represents a symbol of a formal limiting process
    ``limit(r, r->oo)``.

    Direction can be

      * 0                - defines ``undefined``
      * complex number   - defines direction in complex plain
      * undefined        - defines infinity with undefined direction (``zoo``)

    Derived classes may redefine classmethods::

      IsUnbounded(x)        - returns 1 if x is unbounded expression
      IsZero(x)             - returns 1 if x is zero
      EqualArg(x,y)         - returns 1 if Arg(x)==Arg(y).
      
    to add algebra support (including symbolic functions) to direction
    expression.

    The following notation is used::

      +oo = Infinity(1)
      -oo = Infinity(-1)
      undefined = Infinity(0)
      zoo = Infinity(undefined)

    """

    one = 1
    zero = 0

    _add_ops = None

    @classmethod
    def EqualArg(cls, x, y):
        if isinstance(x, numbertypes) and isinstance(y, numbertypes):
            if x and y:
                if isinstance(x, complextypes):
                    if isinstance(y, complextypes):
                        return x.real * y.imag == x.imag * y.real
                    else:
                        return False
                if isinstance(y, complextypes):
                    return False
                return cmp(x,0)==cmp(y,0)
            return False
        if is_zoo(x) or is_zoo(y) or is_undefined(x) or is_undefined(y):
            return False
        if x==y:
            return True
        if not x or not y:
            return False
        if isinstance(x, Expr): algebra = type(x)
        elif isinstance(y, Expr): algebra = type(y)
        else: raise NotImplementedError(`cls, x, y`)
        return algebra.Apply(cls.EqualArg, x, y)

    @classmethod
    def IsUnbounded(cls, x):
        if isinstance(x, Expr):
            head, data = x.pair
            if head is NUMBER:
                return cls.IsUnbounded(data)
            is_bounded = getattr(x, 'is_bounded', None)
            if is_bounded is not None:
                return not is_bounded
            y = x.evalf(2)
            if y.head is NUMBER:
                return cls.IsUnbounded(y.data)
            return APPLY.new(type(x), (cls.IsUnbounded, (x,)))
        if isinstance(x, Infinity):
            return True
        if isinstance(x, numbertypes):
            # XXX: handle mpf, nan, inf
            return False
        if isinstance(x, Expr):
            algebra = type(x)
        else:
            return NotImplemented
        return algebra.Apply(cls.IsUnbounded, x)
    
    @classmethod
    def IsPositive(cls, x):
        if isinstance(x, Expr):
            head, data = x.pair
            if head is NUMBER:
                return cls.IsPositive(data)
            y = x.evalf(2)
            if y.head is NUMBER:
                return cls.IsPositive(y.data)
            return APPLY.new(type(x), (cls.IsPositive, (x,)))
        if isinstance(x, numbertypes):
            return cmp(x, 0)==1
        if isinstance(x, cls):
            return cls.IsPositive(x.data)
        if isinstance(x, Expr): algebra = type(x)
        else: return NotImplemented
        return algebra.Apply(cls.IsPositive, x)

    def __new__(cls, direction):
        obj = object.__new__(cls)
        if isinstance(direction, Expr):
            head, data = direction.pair
            if head is NUMBER:
                return cls(data)
            elif head is TERM_COEFF:
                t, c = data
                if c>0:
                    return cls(t)
            r = direction.get_direction()
            if r is not NotImplemented:
                return cls(r)
        if is_undefined(direction):
            obj.data = direction
        elif isinstance(direction, realtypes):
            obj.data = cmp(direction, 0)
        elif isinstance(direction, complextypes):
            re, im = direction.real, direction.imag
            if re:
                if abs(im) > abs(re):
                    obj.data = mpqc(re, im)/im
                else:
                    obj.data = mpqc(re, im)/re
            elif im:
                obj.data = mpqc(re, im)/im
            else:
                obj.data = 0
        else:
            obj.data = direction
        return obj

    def __repr__(self):
        return '%s(%r)' % (type(self).__name__, self.data)

    def __str__(self):
        d = self.data
        if is_zoo(self):
            return 'zoo'
        if is_undefined(self):
            return 'undefined'
        if d==1:
            return 'oo'
        if d==-1:
            return '-oo'
        a = self._add_ops
        if a is not None:
            lhs, rhs = a
            return '%s + (%s)' % (lhs, rhs)
        return 'oo*(%s)' % (d,)

    def __eq__(self, other):
        if isinstance(other, type(self)):
            return self.data == other.data
        return False

    def __abs__(self):
        """ ``abs(oo*x) -> oo*abs(x)``
        """
        if is_undefined(self):
            return self
        return type(self)(1) # +oo

    def __neg__(self):
        """ negation of extended number
        
        ::

          -(oo*U1(x)) -> oo*(-U1(x))
          -zoo        -> zoo
        """
        if is_zoo(self) or is_undefined(self):
            return self
        return type(self)(-self.data)

    def __add__(self, other):
        """ addition of extended numbers

        ::

          oo*x + oo*y     -> oo*(EqualArg(x, y) * x)
          oo*x + y        -> oo*((1+IsUnbounded(y)*(EqualArg(x,y)-1))*x)
          y + oo*x        -> oo*x + y
        """
        cls = type(self)
        if is_undefined(self) or is_undefined(other) or is_zoo(other):
            return cls(0)
        if is_zoo(self) and isinstance(other, cls):
            return cls(0)
        x = self.data
        if isinstance(other, cls):
            y = other.data
            r = cls(cls.EqualArg(x, y) * x)
            r._add_ops = (self, other)
        else:
            y = other
            y_unbounded = cls.IsUnbounded(y)
            if not y_unbounded:
                return self
            z = (1+y_unbounded*(cls.EqualArg(x, y)-1))
            r = cls(z * x)
            r._add_ops = (self, other)
        return r

    __radd__ = __add__

    def __sub__(self, other):
        """ substraction of extended numbers

        ::

          oo*x - y -> oo*x + (-y)
        """
        return self + (-other)

    def __rsub__(self, other):
        """ substraction of extended numbers

        ::

          y - oo*x -> oo*(-x) + y
        """
        return (-self) + other

    def __mul__(self, other):
        """ multiplication of extended numbers

        ::
        
          (oo*x) * (oo*y) -> oo*(x*y)
          (oo*x) * y          -> oo*(x*y)
        """
        cls = type(self)
        if is_undefined(self) or is_undefined(other) or not other:
            return cls(0)
        if isinstance(other, Infinity):
            return cls(self.data * other.data)
        return cls(self.data * other)

    __rmul__ = __mul__

    def __div__(self, other):
        """ division of extended numbers

        ::

          (oo*x) / (oo*y)     -> undefined
          (oo*x) / 0          -> zoo
          (oo*x) / y          -> oo*(x/y)
        """
        cls = type(self)
        if is_undefined(self) or is_undefined(other) or isinstance(other, cls):
            return cls(0)
        if not other:
            return cls(cls(0))
        if isinstance(other, Expr):
            head, data = other.pair
            if head is NUMBER:
                other = data
            else:
                head, data = other.evalf(2).pair
                if head is NUMBER:
                    other = data
        if isinstance(other, realtypes):
            if other < 0:
                return cls(-self.data)
            return self
        return cls(self.data / other)

    def __rdiv__(self, other):
        """ division of extended numbers

        ::

          y / oo*x -> 0*(1/x)
        """
        if is_undefined(self):
            return self
        return type(other)(0)

    def __pow__(self, other):
        """ exponentiation of extended numbers

        ::

          (oo*x)**0  -> 1
          (oo*x)**(oo*y) -> 0 if y<0; oo*(IsPositive(x)) if y > 0
          (oo*x)**y      -> 0 if y<0; oo*(x**y) if y > 0
        """
        if not other:
            return self.one
        cls = type(self)
        if is_undefined(self) or is_undefined(other) or is_zoo(other):
            return cls(0)
        x = self.data
        zero = self.zero
        algebra = None
        if isinstance(other, Expr):
            algebra = type(other)
            head, data = other.pair
            if head is NUMBER:
                other = data
        if isinstance(other, realtypes):
            if other < 0:
                return zero
            if isinstance(x, realtypes):
                if x > 0:
                    return cls(1)
                return cls((-1)**other)
            return cls(x ** other)
        if isinstance(other, cls):
            y = other.data
            if isinstance(y, realtypes):
                if y < 0:
                    return zero
                if isinstance(x, realtypes):
                    if x > 0:
                        return cls(1)
                    if y==1:
                        return cls(cls(0))
                if is_zoo(self):
                    return self
                return cls(cls.IsPositive(x))
        if algebra is not None:
            exp = algebra(other)
            return algebra(POW, (algebra(SPECIAL, self), exp))
        return NotImplemented

    def __rpow__(self, other):
        """ exponentiation of extended numbers

        ::

          1 ** (oo*x) -> 1
          y ** (oo*x) -> (y**x)**(oo)
        """
        cls = type(self)
        x = self.data
        zero = self.zero
        one = self.one
        algebra = None
        if isinstance(other, Expr):
            algebra = type(other)
            head, data = other.pair
            if head is NUMBER:
                other = data
        if isinstance(other, realtypes):
            if other==1:
                return one
            elif other>1:
                if x==1:
                    return self
                if x==-1:
                    return zero
            elif other>-1:
                if x==1:
                    return zero
                if x==-1:
                    return cls(cls(0))
            elif other==-1:
                return cls(0)
            else:
                if x==1:
                    return cls(cls(0))
                if x==-1:
                    return zero
            if is_zoo(self):
                return cls(0)
            if is_undefined(self):
                return self
        if algebra is not None:
            base = algebra(other)
            return algebra(POW, (base, algebra(SPECIAL, self)))
        return NotImplemented

    def __lt__(self, other):
        if is_undefined(self) or is_undefined(other) or is_zoo(self) or is_zoo(other):
            return False
        x = self.data
        if isinstance(other, type(self)):
            y = other.data
            if x in [-1,1] and y in [-1,1]:
                return x < y
        if isinstance(other, realtypes):
            if x in [-1, 1]:
                return 2*x < cmp(other, 0)
        return NotImplemented

    __le__ = __lt__

    def __gt__(self, other):
        if is_undefined(self) or is_undefined(other) or is_zoo(self) or is_zoo(other):
            return False
        x = self.data
        if isinstance(other, type(self)):
            y = other.data
            if x in [-1,1] and y in [-1,1]:
                return x > y
        if isinstance(other, realtypes):
            if x in [-1, 1]:
                return 2*x > cmp(other, 0)
        return NotImplemented

    __ge__ = __gt__

    @classmethod
    def get_oo(cls):
        return cls(1)

    @classmethod
    def get_moo(cls):
        return cls(-1)

    @classmethod
    def get_zoo(cls):
        return cls(cls(0))

    @classmethod
    def get_undefined(cls):
        return cls(0)

    def is_undefined(self):
        return self.data==0
    



#!/usr/bin/env python
# Parallel Python Software: http://www.parallelpython.com
# Copyright (c) 2005-2012, Vitalii Vanovschi
# All rights reserved.
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#    * Redistributions of source code must retain the above copyright notice,
#      this list of conditions and the following disclaimer.
#    * Redistributions in binary form must reproduce the above copyright
#      notice, this list of conditions and the following disclaimer in the
#      documentation and/or other materials provided with the distribution.
#    * Neither the name of the author nor the names of its contributors
#      may be used to endorse or promote products derived from this software
#      without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
# THE POSSIBILITY OF SUCH DAMAGE.
"""
Parallel Python Software, Network Server

http://www.parallelpython.com - updates, documentation, examples and support
forums
"""

import logging
import getopt
import sys
import socket
import threading
import random
import string
import time
import os

import pp
import ppauto
import ppcommon
import pptransport

copyright = "Copyright (c) 2005-2012 Vitalii Vanovschi. All rights reserved"
version = "1.6.2"

LISTEN_SOCKET_TIMEOUT = 20

# compatibility with Python 2.6
try:
    import hashlib
    sha_new = hashlib.sha1
except ImportError:
    import sha
    sha_new = sha.new


class _NetworkServer(pp.Server):
    """Network Server Class
    """

    def __init__(self, ncpus="autodetect", interface="0.0.0.0",
                broadcast="255.255.255.255", port=None, secret=None,
                timeout=None, restart=False, proto=2, socket_timeout=3600):
        pp.Server.__init__(self, ncpus, (), secret, restart,
                proto, socket_timeout)
        self.host = interface
        self.bcast = broadcast
        if port is not None:
            self.port = port
        else:
            self.port = self.default_port
        self.timeout = timeout
        self.ncon = 0
        self.last_con_time = time.time()
        self.ncon_lock = threading.Lock()

        self.logger.debug("Strarting network server interface=%s port=%i"
                % (self.host, self.port))
        if self.timeout is not None:
            self.logger.debug("ppserver will exit in %i seconds if no "\
                    "connections with clients exist" % (self.timeout))
            ppcommon.start_thread("timeout_check",  self.check_timeout)

    def ncon_add(self, val):
        """Keeps track of the number of connections and time of the last one"""
        self.ncon_lock.acquire()
        self.ncon += val
        self.last_con_time = time.time()
        self.ncon_lock.release()

    def check_timeout(self):
        """Checks if timeout happened and shutdowns server if it did"""
        while True:
            if self.ncon == 0:
                idle_time = time.time() - self.last_con_time
                if idle_time < self.timeout:
                    time.sleep(self.timeout - idle_time)
                else:
                    self.logger.debug("exiting ppserver due to timeout (no client"\
                            " connections in last %i sec)", self.timeout)
                    os._exit(0)
            else:
                time.sleep(self.timeout)

    def listen(self):
        """Initiates listenting to incoming connections"""
        try:
            self.ssocket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            # following allows ppserver to restart faster on the same port
            self.ssocket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
            self.ssocket.settimeout(LISTEN_SOCKET_TIMEOUT)
            self.ssocket.bind((self.host, self.port))
            self.ssocket.listen(5)
        except socket.error, e:
            self.logger.error("Cannot create socket for %s:%s, %s", self.host, self.port, e)

        try:
            while 1:
                csocket = None
                #accept connections from outside
                try:
                    (csocket, address) = self.ssocket.accept()
                except socket.timeout:
                    pass
                if self._exiting:
                    return                
                #now do something with the clientsocket
                #in this case, we'll pretend this is a threaded server
                if csocket:
                    ppcommon.start_thread("client_socket",  self.crun, (csocket,  ))
        except:
            if pp.SHOW_EXPECTED_EXCEPTIONS:
                self.logger.debug("Exception in listen method (possibly expected)", exc_info=True)
            self.logger.debug("Closing server socket")
            self.ssocket.close()            

    def crun(self, csocket):
        """Authenticates client and handles its jobs"""
        mysocket = pptransport.CSocketTransport(csocket, self.socket_timeout)
        #send PP version
        mysocket.send(version)
        #generate a random string
        srandom = "".join([random.choice(string.ascii_letters)
                for i in xrange(16)])
        mysocket.send(srandom)
        answer = sha_new(srandom+self.secret).hexdigest()
        clientanswer = mysocket.receive()
        if answer != clientanswer:
            self.logger.warning("Authentication failed, client host=%s, port=%i"
                    % csocket.getpeername())
            mysocket.send("FAILED")
            csocket.close()
            return
        else:
            mysocket.send("OK")

        ctype = mysocket.receive()
        self.logger.debug("Control message received: " + ctype)
        self.ncon_add(1)
        try:
            if ctype == "STAT":
                #reset time at each new connection
                self.get_stats()["local"].time = 0.0
                mysocket.send(str(self.get_ncpus()))
                while 1:
                    mysocket.receive()
                    mysocket.send(str(self.get_stats()["local"].time))
            elif ctype=="EXEC":
                while 1:
                    sfunc = mysocket.creceive()
                    sargs = mysocket.receive()
                    fun = self.insert(sfunc, sargs)
                    sresult = fun(True)
                    mysocket.send(sresult)
        except:
            if self._exiting:
                return
            if pp.SHOW_EXPECTED_EXCEPTIONS:
                self.logger.debug("Exception in crun method (possibly expected)", exc_info=True)
            self.logger.debug("Closing client socket")
            csocket.close()
            self.ncon_add(-1)

    def broadcast(self):
        """Initiaates auto-discovery mechanism"""
        discover = ppauto.Discover(self)
        ppcommon.start_thread("server_broadcast",  discover.run,
                ((self.host, self.port), (self.bcast, self.port)))


def parse_config(file_loc):
    """
    Parses a config file in a very forgiving way.
    """
    # If we don't have configobj installed then let the user know and exit
    try:
        from configobj import ConfigObj
    except ImportError, ie:
        print >> sys.stderr, ("ERROR: You must have config obj installed to use"
                "configuration files. You can still use command line switches.")
        sys.exit(1)

    if not os.access(file_loc, os.F_OK):
        print >> sys.stderr, "ERROR: Can not access %s." % arg
        sys.exit(1)

    # Load the configuration file
    config = ConfigObj(file_loc)
    # try each config item and use the result if it exists. If it doesn't
    # then simply pass and move along
    try:
        args['secret'] = config['general'].get('secret')
    except:
        pass

    try:
        autodiscovery = config['network'].as_bool('autodiscovery')
    except:
        pass

    try:
        args['interface'] = config['network'].get('interface',
                                                  default="0.0.0.0")
    except:
        pass

    try:
        args['broadcast'] = config['network'].get('broadcast')
    except:
        pass

    try:
        args['port'] = config['network'].as_int('port')
    except:
        pass

    try:
        args['loglevel'] = config['general'].as_bool('debug')
    except:
        pass

    try:
        args['ncpus'] = config['general'].as_int('workers')
    except:
        pass

    try:
        args['proto'] = config['general'].as_int('proto')
    except:
        pass

    try:
        args['restart'] = config['general'].as_bool('restart')
    except:
        pass

    try:
        args['timeout'] = config['network'].as_int('timeout')
    except:
        pass

    try:
        args['socket_timeout'] = config['network'].as_int('socket_timeout')
    except:
        pass
    # Return a tuple of the args dict and autodiscovery variable
    return args, autodiscovery


def print_usage():
    """Prints help"""
    print "Parallel Python Network Server (pp-" + version + ")"
    print "Usage: ppserver.py [-hdar] [-f format] [-n proto]"\
            " [-c config_path] [-i interface] [-b broadcast]"\
            " [-p port] [-w nworkers] [-s secret] [-t seconds]"\
            " [-k seconds]"
    print
    print "Options: "
    print "-h                 : this help message"
    print "-d                 : set log level to debug"
    print "-f format          : log format"
    print "-a                 : enable auto-discovery service"
    print "-r                 : restart worker process after each"\
            " task completion"
    print "-n proto           : protocol number for pickle module"
    print "-c path            : path to config file"
    print "-i interface       : interface to listen"
    print "-b broadcast       : broadcast address for auto-discovery service"
    print "-p port            : port to listen"
    print "-w nworkers        : number of workers to start"
    print "-s secret          : secret for authentication"
    print "-t seconds         : timeout to exit if no connections with "\
            "clients exist"
    print "-k seconds         : socket timeout in seconds"
    print
    print "Due to the security concerns always use a non-trivial secret key."
    print "Secret key set by -s switch will override secret key assigned by"
    print "pp_secret variable in .pythonrc.py"
    print
    print "Please visit http://www.parallelpython.com for extended up-to-date"
    print "documentation, examples and support forums"


def create_network_server(argv):
    try:
        opts, args = getopt.getopt(argv, "hdarn:c:b:i:p:w:s:t:f:k:", ["help"])
    except getopt.GetoptError:
        print_usage()
        raise

    args = {}
    autodiscovery = False

    log_level = logging.WARNING
    log_format = "%(asctime)s - %(name)s - %(levelname)s - %(message)s"

    for opt, arg in opts:
        if opt in ("-h", "--help"):
            print_usage()
            sys.exit()
        elif opt == "-c":
            args, autodiscovery = parse_config(arg)
        elif opt == "-d":
            log_level = logging.DEBUG
            pp.SHOW_EXPECTED_EXCEPTIONS = True
        elif opt == "-f":
            log_format = arg
        elif opt == "-i":
            args["interface"] = arg
        elif opt == "-s":
            args["secret"] = arg
        elif opt == "-p":
            args["port"] = int(arg)
        elif opt == "-w":
            args["ncpus"] = int(arg)
        elif opt == "-a":
            autodiscovery = True
        elif opt == "-r":
            args["restart"] = True
        elif opt == "-b":
            args["broadcast"] = arg
        elif opt == "-n":
            args["proto"] = int(arg)
        elif opt == "-t":
            args["timeout"] = int(arg)
        elif opt == "-k":
            args["socket_timeout"] = int(arg)

    log_handler = logging.StreamHandler()
    log_handler.setFormatter(logging.Formatter(log_format))
    logging.getLogger("pp").setLevel(log_level)
    logging.getLogger("pp").addHandler(log_handler)

    server = _NetworkServer(**args)
    if autodiscovery:
        server.broadcast()
    return server    
    

if __name__ == "__main__":
    server = create_network_server(sys.argv[1:])
    server.listen()
    #have to destroy it here explicitly otherwise an exception
    #comes out in Python 2.4
    del server
    

# Parallel Python Software: http://www.parallelpython.com
